@@ -13,18 +13,9 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <string.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <signal.h>
 #include <assert.h>
-#include <time.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-
 
 #include "hostapd.h"
 #include "radius.h"
@@ -397,8 +388,7 @@ accounting_receive(struct radius_msg *msg, struct radius_msg *req,
 		return RADIUS_RX_UNKNOWN;
 	}
 
-	if (radius_msg_verify_acct(msg, shared_secret, shared_secret_len, req))
-	{
+	if (radius_msg_verify(msg, shared_secret, shared_secret_len, req, 0)) {
 		printf("Incoming RADIUS packet did not have correct "
 		       "Authenticator - dropped\n");
 		return RADIUS_RX_INVALID_AUTHENTICATOR;
