@@ -57,22 +57,8 @@ struct sta_info {
 
 	u8 *challenge; /* IEEE 802.11 Shared Key Authentication Challenge */
 
-	int pairwise; /* Pairwise cipher suite, WPA_CIPHER_* */
-	u8 *wpa_ie;
-	size_t wpa_ie_len;
 	struct wpa_state_machine *wpa_sm;
-	enum {
-		WPA_VERSION_NO_WPA = 0 /* WPA not used */,
-		WPA_VERSION_WPA = 1 /* WPA / IEEE 802.11i/D3.0 */,
-		WPA_VERSION_WPA2 = 2 /* WPA2 / IEEE 802.11i */
-	} wpa;
-	int wpa_key_mgmt; /* the selected WPA_KEY_MGMT_* */
-	struct rsn_pmksa_cache *pmksa;
 	struct rsn_preauth_interface *preauth_iface;
-	u8 req_replay_counter[8 /* WPA_REPLAY_COUNTER_LEN */];
-	int req_replay_counter_used;
-	u32 dot11RSNAStatsTKIPLocalMICFailures;
-	u32 dot11RSNAStatsTKIPRemoteMICFailures;
 };
 
 
