@@ -13,15 +13,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <string.h>
-#include <sys/socket.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include "includes.h"
 #include <grp.h>
 
 #include "hostapd.h"
@@ -361,6 +353,11 @@ static int hostapd_config_read_eap_user(const char *fname,
 			}
 			memcpy(user->identity, start, pos - start);
 			user->identity_len = pos - start;
+
+			if (pos[0] == '"' && pos[1] == '*') {
+				user->wildcard_prefix = 1;
+				pos++;
+			}
 		}
 		pos++;
 		while (*pos == ' ' || *pos == '\t')
@@ -451,6 +448,31 @@ static int hostapd_config_read_eap_user(const char *fname,
 			user->password_len = pos - start;
 
 			pos++;
+		} else if (strncmp(pos, "hash:", 5) == 0) {
+			pos += 5;
+			pos2 = pos;
+			while (*pos2 != '\0' && *pos2 != ' ' &&
+			       *pos2 != '\t' && *pos2 != '#')
+				pos2++;
+			if (pos2 - pos != 32) {
+				printf("Invalid password hash on line %d in "
+				       "'%s'\n", line, fname);
+				goto failed;
+			}
+			user->password = malloc(16);
+			if (user->password == NULL) {
+				printf("Failed to allocate memory for EAP "
+				       "password hash\n");
+				goto failed;
+			}
+			if (hexstr2bin(pos, user->password, 16) < 0) {
+				printf("Invalid hash password on line %d in "
+				       "'%s'\n", line, fname);
+				goto failed;
+			}
+			user->password_len = 16;
+			user->password_hash = 1;
+			pos = pos2;
 		} else {
 			pos2 = pos;
 			while (*pos2 != '\0' && *pos2 != ' ' &&
@@ -494,6 +516,7 @@ static int hostapd_config_read_eap_user(const char *fname,
 
 	failed:
 		if (user) {
+			free(user->password);
 			free(user->identity);
 			free(user);
 		}
@@ -1001,6 +1024,10 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 		} else if (strcmp(buf, "rsn_preauth_interfaces") == 0) {
 			conf->rsn_preauth_interfaces = strdup(pos);
 #endif /* CONFIG_RSN_PREAUTH */
+#ifdef CONFIG_STAKEY
+		} else if (strcmp(buf, "stakey") == 0) {
+			conf->stakey = atoi(pos);
+#endif /* CONFIG_STAKEY */
 		} else if (strcmp(buf, "ctrl_interface") == 0) {
 			free(conf->ctrl_interface);
 			conf->ctrl_interface = strdup(pos);
@@ -1206,6 +1233,15 @@ hostapd_get_eap_user(const struct hostapd_config *conf, const u8 *identity,
 			/* Wildcard match */
 			break;
 		}
+
+		if (!phase2 && user->wildcard_prefix &&
+		    identity_len >= user->identity_len &&
+		    memcmp(user->identity, identity, user->identity_len) == 0)
+		{
+			/* Wildard prefix match */
+			break;
+		}
+
 		if (user->phase2 == !!phase2 &&
 		    user->identity_len == identity_len &&
 		    memcmp(user->identity, identity, identity_len) == 0)
