@@ -25,6 +25,9 @@ struct hostapd_eap_user {
 	size_t password_len;
 	int phase2;
 	int force_version;
+	unsigned int wildcard_prefix:1;
+	unsigned int password_hash:1; /* whether password is hashed with
+				       * nt_password_hash() */
 };
 
 struct hostapd_config {
@@ -122,6 +125,7 @@ struct hostapd_config {
 	int wpa_gmk_rekey;
 	int rsn_preauth;
 	char *rsn_preauth_interfaces;
+	int stakey;
 
 	char *ctrl_interface; /* directory for UNIX domain sockets */
 	gid_t ctrl_interface_gid;
