@@ -13,15 +13,8 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <netinet/in.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 
 #ifdef USE_KERNEL_HEADERS
 #include <asm/types.h>
@@ -42,6 +35,7 @@
 #include "priv_netlink.h"
 #include "ieee802_11.h"
 #include "sta_info.h"
+#include "hostap_common.h"
 
 
 struct hostap_driver_data {
@@ -52,6 +46,8 @@ struct hostap_driver_data {
 	int sock; /* raw packet socket for driver access */
 	int ioctl_sock; /* socket for ioctl() use */
 	int wext_sock; /* socket for wireless events */
+
+	int we_version;
 };
 
 static const struct driver_ops hostap_driver_ops;
@@ -198,12 +194,8 @@ static void handle_frame(hostapd *hapd, u8 *buf, size_t len)
 
 	if (type != WLAN_FC_TYPE_MGMT || stype != WLAN_FC_STYPE_BEACON ||
 	    hapd->conf->debug >= HOSTAPD_DEBUG_EXCESSIVE) {
-		HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE,
-			      "Received %lu bytes management frame\n",
-			      (unsigned long) len);
-		if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MSGDUMPS)) {
-			hostapd_hexdump("RX frame", buf, len);
-		}
+		wpa_hexdump(MSG_MSGDUMP, "Received management frrame",
+			    buf, len);
 	}
 
 	ver = fc & WLAN_FC_PVER;
@@ -334,8 +326,8 @@ static int hostap_send_mgmt_frame(void *priv, const void *msg, size_t len,
 }
 
 
-static int hostap_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len,
-			     int encrypt)
+static int hostap_send_eapol(void *priv, const u8 *addr, const u8 *data,
+			     size_t data_len, int encrypt)
 {
 	struct hostap_driver_data *drv = priv;
 	struct ieee80211_hdr *hdr;
@@ -383,7 +375,8 @@ static int hostap_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len,
 }
 
 
-static int hostap_set_sta_authorized(void *priv, u8 *addr, int authorized)
+static int hostap_set_sta_authorized(void *priv, const u8 *addr,
+				     int authorized)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
@@ -467,8 +460,8 @@ static int hostapd_ioctl(void *priv, struct prism2_hostapd_param *param,
 }
 
 
-static int hostap_set_encryption(void *priv, const char *alg, u8 *addr,
-				 int idx, u8 *key, size_t key_len)
+static int hostap_set_encryption(void *priv, const char *alg, const u8 *addr,
+				 int idx, const u8 *key, size_t key_len)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param *param;
@@ -504,7 +497,7 @@ static int hostap_set_encryption(void *priv, const char *alg, u8 *addr,
 }
 
 
-static int hostap_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
+static int hostap_get_seqnum(void *priv, const u8 *addr, int idx, u8 *seq)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param *param;
@@ -595,7 +588,7 @@ static int hostap_set_privacy(void *priv, int enabled)
 }
 
  
-static int hostap_set_ssid(void *priv, u8 *buf, int len)
+static int hostap_set_ssid(void *priv, const u8 *buf, int len)
 {
 	struct hostap_driver_data *drv = priv;
 	struct iwreq iwr;
@@ -628,7 +621,8 @@ static int hostap_flush(void *priv)
 
 
 static int hostap_read_sta_data(void *priv,
-				struct hostap_sta_driver_data *data, u8 *addr)
+				struct hostap_sta_driver_data *data,
+				const u8 *addr)
 {
 	struct hostap_driver_data *drv = priv;
 	char buf[1024], line[128], *pos;
@@ -668,7 +662,7 @@ static int hostap_read_sta_data(void *priv,
 }
 
 
-static int hostap_sta_add(void *priv, u8 *addr, u16 aid, u16 capability,
+static int hostap_sta_add(void *priv, const u8 *addr, u16 aid, u16 capability,
 			  u8 tx_supp_rates)
 {
 	struct hostap_driver_data *drv = priv;
@@ -684,7 +678,7 @@ static int hostap_sta_add(void *priv, u8 *addr, u16 aid, u16 capability,
 }
 
 
-static int hostap_sta_remove(void *priv, u8 *addr)
+static int hostap_sta_remove(void *priv, const u8 *addr)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
@@ -702,7 +696,7 @@ static int hostap_sta_remove(void *priv, u8 *addr)
 }
 
 
-static int hostap_get_inact_sec(void *priv, u8 *addr)
+static int hostap_get_inact_sec(void *priv, const u8 *addr)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
@@ -718,7 +712,7 @@ static int hostap_get_inact_sec(void *priv, u8 *addr)
 }
 
 
-static int hostap_sta_clear_stats(void *priv, u8 *addr)
+static int hostap_sta_clear_stats(void *priv, const u8 *addr)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
@@ -734,7 +728,7 @@ static int hostap_sta_clear_stats(void *priv, u8 *addr)
 }
 
 
-static int hostap_set_assoc_ap(void *priv, u8 *addr)
+static int hostap_set_assoc_ap(void *priv, const u8 *addr)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
@@ -810,21 +804,37 @@ static void hostapd_wireless_event_wireless(struct hostap_driver_data *drv,
 					    char *data, int len)
 {
 	struct hostapd_data *hapd = drv->hapd;
-	struct iw_event *iwe;
+	struct iw_event iwe_buf, *iwe = &iwe_buf;
 	char *pos, *end, *custom, *buf;
 
 	pos = data;
 	end = data + len;
 
 	while (pos + IW_EV_LCP_LEN <= end) {
-		iwe = (struct iw_event *) pos;
+		/* Event data may be unaligned, so make a local, aligned copy
+		 * before processing. */
+		memcpy(&iwe_buf, pos, IW_EV_LCP_LEN);
 		HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE, "Wireless event: "
 			      "cmd=0x%x len=%d\n", iwe->cmd, iwe->len);
 		if (iwe->len <= IW_EV_LCP_LEN)
 			return;
+
+		custom = pos + IW_EV_POINT_LEN;
+		if (drv->we_version > 18 &&
+		    (iwe->cmd == IWEVMICHAELMICFAILURE ||
+		     iwe->cmd == IWEVCUSTOM)) {
+			/* WE-19 removed the pointer from struct iw_point */
+			char *dpos = (char *) &iwe_buf.u.data.length;
+			int dlen = dpos - (char *) &iwe_buf;
+			memcpy(dpos, pos + IW_EV_LCP_LEN,
+			       sizeof(struct iw_event) - dlen);
+		} else {
+			memcpy(&iwe_buf, pos, sizeof(struct iw_event));
+			custom += IW_EV_POINT_OFF;
+		}
+
 		switch (iwe->cmd) {
 		case IWEVCUSTOM:
-			custom = pos + IW_EV_POINT_LEN;
 			if (custom + iwe->u.data.length > end)
 				return;
 			buf = malloc(iwe->u.data.length + 1);
@@ -926,12 +936,60 @@ static void hostapd_wireless_event_receive(int sock, void *eloop_ctx,
 }
 
 
+static int hostap_get_we_version(struct hostap_driver_data *drv)
+{
+	struct iw_range *range;
+	struct iwreq iwr;
+	int minlen;
+	size_t buflen;
+
+	drv->we_version = 0;
+
+	/*
+	 * Use larger buffer than struct iw_range in order to allow the
+	 * structure to grow in the future.
+	 */
+	buflen = sizeof(struct iw_range) + 500;
+	range = malloc(buflen);
+	if (range == NULL)
+		return -1;
+	memset(range, 0, buflen);
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
+	iwr.u.data.pointer = (caddr_t) range;
+	iwr.u.data.length = buflen;
+
+	minlen = ((char *) &range->enc_capa) - (char *) range +
+		sizeof(range->enc_capa);
+
+	if (ioctl(drv->ioctl_sock, SIOCGIWRANGE, &iwr) < 0) {
+		perror("ioctl[SIOCGIWRANGE]");
+		free(range);
+		return -1;
+	} else if (iwr.u.data.length >= minlen &&
+		   range->we_version_compiled >= 18) {
+		wpa_printf(MSG_DEBUG, "SIOCGIWRANGE: WE(compiled)=%d "
+			   "WE(source)=%d enc_capa=0x%x",
+			   range->we_version_compiled,
+			   range->we_version_source,
+			   range->enc_capa);
+		drv->we_version = range->we_version_compiled;
+	}
+
+	free(range);
+	return 0;
+}
+
+
 static int hostap_wireless_event_init(void *priv)
 {
 	struct hostap_driver_data *drv = priv;
 	int s;
 	struct sockaddr_nl local;
 
+	hostap_get_we_version(drv);
+
 	drv->wext_sock = -1;
 
 	s = socket(PF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
@@ -1030,7 +1088,7 @@ static void hostap_driver_deinit(void *priv)
 }
 
 
-static int hostap_sta_deauth(void *priv, u8 *addr, int reason)
+static int hostap_sta_deauth(void *priv, const u8 *addr, int reason)
 {
 	struct hostap_driver_data *drv = priv;
 	struct ieee80211_mgmt mgmt;
@@ -1047,7 +1105,7 @@ static int hostap_sta_deauth(void *priv, u8 *addr, int reason)
 }
 
 
-static int hostap_sta_disassoc(void *priv, u8 *addr, int reason)
+static int hostap_sta_disassoc(void *priv, const u8 *addr, int reason)
 {
 	struct hostap_driver_data *drv = priv;
 	struct ieee80211_mgmt mgmt;
