@@ -33,30 +33,31 @@ struct driver_ops {
 	 */
 	int (*set_privacy)(void *priv, int enabled);
 
-	int (*set_encryption)(void *priv, const char *alg, u8 *addr,
-			      int idx, u8 *key, size_t key_len);
-	int (*get_seqnum)(void *priv, u8 *addr, int idx, u8 *seq);
+	int (*set_encryption)(void *priv, const char *alg, const u8 *addr,
+			      int idx, const u8 *key, size_t key_len);
+	int (*get_seqnum)(void *priv, const u8 *addr, int idx, u8 *seq);
 	int (*flush)(void *priv);
 	int (*set_generic_elem)(void *priv, const u8 *elem, size_t elem_len);
 
 	int (*read_sta_data)(void *priv, struct hostap_sta_driver_data *data,
-			     u8 *addr);
-	int (*send_eapol)(void *priv, u8 *addr, u8 *data, size_t data_len,
-			  int encrypt);
-	int (*set_sta_authorized)(void *driver, u8 *addr, int authorized);
-	int (*sta_deauth)(void *priv, u8 *addr, int reason);
-	int (*sta_disassoc)(void *priv, u8 *addr, int reason);
-	int (*sta_remove)(void *priv, u8 *addr);
+			     const u8 *addr);
+	int (*send_eapol)(void *priv, const u8 *addr, const u8 *data,
+			  size_t data_len, int encrypt);
+	int (*set_sta_authorized)(void *driver, const u8 *addr,
+				  int authorized);
+	int (*sta_deauth)(void *priv, const u8 *addr, int reason);
+	int (*sta_disassoc)(void *priv, const u8 *addr, int reason);
+	int (*sta_remove)(void *priv, const u8 *addr);
 	int (*get_ssid)(void *priv, u8 *buf, int len);
-	int (*set_ssid)(void *priv, u8 *buf, int len);
+	int (*set_ssid)(void *priv, const u8 *buf, int len);
 	int (*set_countermeasures)(void *priv, int enabled);
 	int (*send_mgmt_frame)(void *priv, const void *msg, size_t len,
 			       int flags);
-	int (*set_assoc_ap)(void *priv, u8 *addr);
-	int (*sta_add)(void *priv, u8 *addr, u16 aid, u16 capability,
+	int (*set_assoc_ap)(void *priv, const u8 *addr);
+	int (*sta_add)(void *priv, const u8 *addr, u16 aid, u16 capability,
 		       u8 tx_supp_rates);
-	int (*get_inact_sec)(void *priv, u8 *addr);
-	int (*sta_clear_stats)(void *priv, u8 *addr);
+	int (*get_inact_sec)(void *priv, const u8 *addr);
+	int (*sta_clear_stats)(void *priv, const u8 *addr);
 };
 
 static inline int
@@ -110,8 +111,8 @@ hostapd_set_privacy(struct hostapd_data *hapd, int enabled)
 }
 
 static inline int
-hostapd_set_encryption(struct hostapd_data *hapd, const char *alg, u8 *addr,
-		       int idx, u8 *key, size_t key_len)
+hostapd_set_encryption(struct hostapd_data *hapd, const char *alg,
+		       const u8 *addr, int idx, u8 *key, size_t key_len)
 {
 	if (hapd->driver == NULL || hapd->driver->set_encryption == NULL)
 		return 0;
@@ -120,7 +121,7 @@ hostapd_set_encryption(struct hostapd_data *hapd, const char *alg, u8 *addr,
 }
 
 static inline int
-hostapd_get_seqnum(struct hostapd_data *hapd, u8 *addr, int idx, u8 *seq)
+hostapd_get_seqnum(struct hostapd_data *hapd, const u8 *addr, int idx, u8 *seq)
 {
 	if (hapd->driver == NULL || hapd->driver->get_seqnum == NULL)
 		return 0;
@@ -146,7 +147,7 @@ hostapd_set_generic_elem(struct hostapd_data *hapd, const u8 *elem,
 
 static inline int
 hostapd_read_sta_data(struct hostapd_data *hapd,
-		      struct hostap_sta_driver_data *data, u8 *addr)
+		      struct hostap_sta_driver_data *data, const u8 *addr)
 {
 	if (hapd->driver == NULL || hapd->driver->read_sta_data == NULL)
 		return -1;
@@ -154,7 +155,7 @@ hostapd_read_sta_data(struct hostapd_data *hapd,
 }
 
 static inline int
-hostapd_send_eapol(struct hostapd_data *hapd, u8 *addr, u8 *data,
+hostapd_send_eapol(struct hostapd_data *hapd, const u8 *addr, const u8 *data,
 		   size_t data_len, int encrypt)
 {
 	if (hapd->driver == NULL || hapd->driver->send_eapol == NULL)
@@ -164,7 +165,8 @@ hostapd_send_eapol(struct hostapd_data *hapd, u8 *addr, u8 *data,
 }
 
 static inline int
-hostapd_set_sta_authorized(struct hostapd_data *hapd, u8 *addr, int authorized)
+hostapd_set_sta_authorized(struct hostapd_data *hapd, const u8 *addr,
+			   int authorized)
 {
 	if (hapd->driver == NULL || hapd->driver->set_sta_authorized == NULL)
 		return 0;
@@ -173,7 +175,7 @@ hostapd_set_sta_authorized(struct hostapd_data *hapd, u8 *addr, int authorized)
 }
 
 static inline int
-hostapd_sta_deauth(struct hostapd_data *hapd, u8 *addr, int reason)
+hostapd_sta_deauth(struct hostapd_data *hapd, const u8 *addr, int reason)
 {
 	if (hapd->driver == NULL || hapd->driver->sta_deauth == NULL)
 		return 0;
@@ -181,7 +183,7 @@ hostapd_sta_deauth(struct hostapd_data *hapd, u8 *addr, int reason)
 }
 
 static inline int
-hostapd_sta_disassoc(struct hostapd_data *hapd, u8 *addr, int reason)
+hostapd_sta_disassoc(struct hostapd_data *hapd, const u8 *addr, int reason)
 {
 	if (hapd->driver == NULL || hapd->driver->sta_disassoc == NULL)
 		return 0;
@@ -189,7 +191,7 @@ hostapd_sta_disassoc(struct hostapd_data *hapd, u8 *addr, int reason)
 }
 
 static inline int
-hostapd_sta_remove(struct hostapd_data *hapd, u8 *addr)
+hostapd_sta_remove(struct hostapd_data *hapd, const u8 *addr)
 {
 	if (hapd->driver == NULL || hapd->driver->sta_remove == NULL)
 		return 0;
@@ -205,7 +207,7 @@ hostapd_get_ssid(struct hostapd_data *hapd, u8 *buf, size_t len)
 }
 
 static inline int
-hostapd_set_ssid(struct hostapd_data *hapd, u8 *buf, size_t len)
+hostapd_set_ssid(struct hostapd_data *hapd, const u8 *buf, size_t len)
 {
 	if (hapd->driver == NULL || hapd->driver->set_ssid == NULL)
 		return 0;
@@ -222,7 +224,7 @@ hostapd_send_mgmt_frame(struct hostapd_data *hapd, const void *msg, size_t len,
 }
 
 static inline int
-hostapd_set_assoc_ap(struct hostapd_data *hapd, u8 *addr)
+hostapd_set_assoc_ap(struct hostapd_data *hapd, const u8 *addr)
 {
 	if (hapd->driver == NULL || hapd->driver->set_assoc_ap == NULL)
 		return 0;
@@ -238,8 +240,8 @@ hostapd_set_countermeasures(struct hostapd_data *hapd, int enabled)
 }
 
 static inline int
-hostapd_sta_add(struct hostapd_data *hapd, u8 *addr, u16 aid, u16 capability,
-		u8 tx_supp_rates)
+hostapd_sta_add(struct hostapd_data *hapd, const u8 *addr, u16 aid,
+		u16 capability, u8 tx_supp_rates)
 {
 	if (hapd->driver == NULL || hapd->driver->sta_add == NULL)
 		return 0;
@@ -248,7 +250,7 @@ hostapd_sta_add(struct hostapd_data *hapd, u8 *addr, u16 aid, u16 capability,
 }
 
 static inline int
-hostapd_get_inact_sec(struct hostapd_data *hapd, u8 *addr)
+hostapd_get_inact_sec(struct hostapd_data *hapd, const u8 *addr)
 {
 	if (hapd->driver == NULL || hapd->driver->get_inact_sec == NULL)
 		return 0;
@@ -261,7 +263,7 @@ void driver_unregister(const char *name);
 const struct driver_ops *driver_lookup(const char *name);
 
 static inline int
-hostapd_sta_clear_stats(struct hostapd_data *hapd, u8 *addr)
+hostapd_sta_clear_stats(struct hostapd_data *hapd, const u8 *addr)
 {
 	if (hapd->driver == NULL || hapd->driver->sta_clear_stats == NULL)
 		return 0;
