@@ -12,16 +12,11 @@
  *
  * See README and COPYING for more details.
  */
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
-#include <sys/ioctl.h>
-#include <errno.h>
 
-#include <sys/socket.h>
+#include "includes.h"
+#include <sys/ioctl.h>
+
 #include <net/if.h>
-#include <netinet/in.h>
 
 #include <net80211/ieee80211.h>
 #include <net80211/ieee80211_crypto.h>
@@ -53,7 +48,7 @@ struct bsd_driver_data {
 
 static const struct driver_ops bsd_driver_ops;
 
-static int bsd_sta_deauth(void *priv, u8 *addr, int reason_code);
+static int bsd_sta_deauth(void *priv, const u8 *addr, int reason_code);
 
 static int
 set80211var(struct bsd_driver_data *drv, int op, const void *arg, int arg_len)
@@ -304,7 +299,7 @@ bsd_set_privacy(void *priv, int enabled)
 }
 
 static int
-bsd_set_sta_authorized(void *priv, u8 *addr, int authorized)
+bsd_set_sta_authorized(void *priv, const u8 *addr, int authorized)
 {
 	struct bsd_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -324,7 +319,7 @@ bsd_set_sta_authorized(void *priv, u8 *addr, int authorized)
 }
 
 static int
-bsd_del_key(void *priv, unsigned char *addr, int key_idx)
+bsd_del_key(void *priv, const u8 *addr, int key_idx)
 {
 	struct bsd_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -347,8 +342,8 @@ bsd_del_key(void *priv, unsigned char *addr, int key_idx)
 
 static int
 bsd_set_key(void *priv, const char *alg,
-	     unsigned char *addr, int key_idx,
-	     u8 *key, size_t key_len)
+	     const u8 *addr, int key_idx,
+	     const u8 *key, size_t key_len)
 {
 	struct bsd_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -398,7 +393,7 @@ bsd_set_key(void *priv, const char *alg,
 
 
 static int
-bsd_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
+bsd_get_seqnum(void *priv, const u8 *addr, int idx, u8 *seq)
 {
 	struct bsd_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -451,7 +446,7 @@ bsd_flush(void *priv)
 
 static int
 bsd_read_sta_driver_data(void *priv, struct hostap_sta_driver_data *data,
-					u8 *addr)
+			 const u8 *addr)
 {
 	struct bsd_driver_data *drv = priv;
 	struct ieee80211req_sta_stats stats;
@@ -478,7 +473,7 @@ bsd_set_opt_ie(void *priv, const u8 *ie, size_t ie_len)
 }
 
 static int
-bsd_sta_deauth(void *priv, u8 *addr, int reason_code)
+bsd_sta_deauth(void *priv, const u8 *addr, int reason_code)
 {
 	struct bsd_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -495,7 +490,7 @@ bsd_sta_deauth(void *priv, u8 *addr, int reason_code)
 }
 
 static int
-bsd_sta_disassoc(void *priv, u8 *addr, int reason_code)
+bsd_sta_disassoc(void *priv, const u8 *addr, int reason_code)
 {
 	struct bsd_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -524,9 +519,9 @@ bsd_del_sta(struct bsd_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 	if (sta != NULL) {
 		sta->flags &= ~WLAN_STA_ASSOC;
 		if (conf->wpa)
-			wpa_sm_event(hapd, sta, WPA_DISASSOC);
+			wpa_auth_sm_event(sta->wpa_sm, WPA_DISASSOC);
 		sta->acct_terminate_cause = RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-		ieee802_1x_set_port_enabled(hapd, sta, 0);
+		ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 		ap_free_sta(hapd, sta);
 	}
 	return 0;
@@ -563,29 +558,20 @@ bsd_new_sta(struct bsd_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 			return -1;		/* XXX not right */
 		}
 		ielen += 2;
-		res = wpa_validate_wpa_ie(hapd, sta, ie.wpa_ie, ielen,
-				ie.wpa_ie[0] == WLAN_EID_RSN ?
-				    HOSTAPD_WPA_VERSION_WPA2 :
-				    HOSTAPD_WPA_VERSION_WPA);
+		if (sta->wpa_sm == NULL)
+			sta->wpa_sm = wpa_auth_sta_init(hapd->wpa_auth,
+							sta->addr);
+		if (sta->wpa_sm == NULL) {
+			printf("Failed to initialize WPA state machine\n");
+			return -1;
+		}
+		res = wpa_validate_wpa_ie(hapd->wpa_auth, sta->wpa_sm,
+					  ie.wpa_ie, ielen);
 		if (res != WPA_IE_OK) {
 			printf("WPA/RSN information element rejected? "
 				"(res %u)\n", res);
 			return -1;
 		}
-		if (sta->wpa_ie != NULL)
-			free(sta->wpa_ie);
-		sta->wpa_ie = malloc(ielen);
-		if (sta->wpa_ie == NULL) {
-			printf("No memory for WPA/RSN information element!\n");
-			return -1;
-		}
-		memcpy(sta->wpa_ie, ie.wpa_ie, ielen);
-		sta->wpa_ie_len = ielen;
-	} else {
-		if (sta->wpa_ie != NULL)
-			free(sta->wpa_ie);
-		sta->wpa_ie = NULL;
-		sta->wpa_ie_len = 0;
 	}
 
 	/*
@@ -594,7 +580,7 @@ bsd_new_sta(struct bsd_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 	 */
 	new_assoc = (sta->flags & WLAN_STA_ASSOC) == 0;
 	sta->flags |= WLAN_STA_ASSOC;
-	wpa_sm_event(hapd, sta, WPA_ASSOC);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_ASSOC);
 	hostapd_new_assoc_sta(hapd, sta, !new_assoc);
 	ieee802_1x_notify_port_enabled(sta->eapol_sm, 1);
 	return 0;
@@ -699,10 +685,10 @@ bsd_wireless_event_deinit(void *priv)
 
 
 static int
-bsd_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len, int encrypt)
+bsd_send_eapol(void *priv, const u8 *addr, const u8 *data, size_t data_len,
+	       int encrypt)
 {
 	struct bsd_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
 	unsigned char buf[3000];
 	unsigned char *bp = buf;
 	struct l2_ethhdr *eth;
@@ -730,8 +716,7 @@ bsd_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len, int encrypt)
 	eth->h_proto = htons(ETH_P_EAPOL);
 	memcpy(eth+1, data, data_len);
 
-	if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MSGDUMPS))
-		hostapd_hexdump("TX EAPOL", bp, len);
+	wpa_hexdump(MSG_MSGDUMP, "TX EAPOL", bp, len);
 
 	status = l2_packet_send(drv->sock_xmit, addr, ETH_P_EAPOL, bp, len);
 
@@ -772,7 +757,7 @@ bsd_get_ssid(void *priv, u8 *buf, int len)
 }
 
 static int
-bsd_set_ssid(void *priv, u8 *buf, int len)
+bsd_set_ssid(void *priv, const u8 *buf, int len)
 {
 	struct bsd_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
