@@ -2,6 +2,7 @@
  * Host AP - driver interaction with MADWIFI 802.11 driver
  * Copyright (c) 2004, Sam Leffler <sam@errno.com>
  * Copyright (c) 2004, Video54 Technologies
+ * Copyright (c) 2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -12,14 +13,10 @@
  *
  * See README and COPYING for more details.
  */
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
+
+#include "includes.h"
+#include <net/if.h>
 #include <sys/ioctl.h>
-#include <errno.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 
 #include <include/compat.h>
 #include <net80211/ieee80211.h>
@@ -31,10 +28,14 @@
 #include <net80211/ieee80211_crypto.h>
 #include <net80211/ieee80211_ioctl.h>
 
+#ifdef IEEE80211_IOCTL_SETWMMPARAMS
+/* Assume this is built against madwifi-ng */
+#define MADWIFI_NG
+#endif /* IEEE80211_IOCTL_SETWMMPARAMS */
+
 #include <net/if_arp.h>
 #include "wireless_copy.h"
 
-#include <netinet/in.h>
 #include <netpacket/packet.h>
 
 #include "hostapd.h"
@@ -71,12 +72,11 @@ struct madwifi_driver_data {
 
 static const struct driver_ops madwifi_driver_ops;
 
-static int madwifi_sta_deauth(void *priv, u8 *addr, int reason_code);
+static int madwifi_sta_deauth(void *priv, const u8 *addr, int reason_code);
 
 static int
 set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 {
-#define	N(a)	(sizeof(a)/sizeof(a[0]))
 	struct iwreq iwr;
 
 	memset(&iwr, 0, sizeof(iwr));
@@ -97,6 +97,32 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 	}
 
 	if (ioctl(drv->ioctl_sock, op, &iwr) < 0) {
+#ifdef MADWIFI_NG
+		int first = IEEE80211_IOCTL_SETPARAM;
+		int last = IEEE80211_IOCTL_GETMODE;
+		static const char *opnames[] = {
+			"ioctl[IEEE80211_IOCTL_SETPARAM]",
+			"ioctl[IEEE80211_IOCTL_GETPARAM]",
+			"ioctl[IEEE80211_IOCTL_SETKEY]",
+			"ioctl[IEEE80211_IOCTL_SETWMMPARAMS]",
+			"ioctl[IEEE80211_IOCTL_DELKEY]",
+			"ioctl[IEEE80211_IOCTL_GETWMMPARAMS]",
+			"ioctl[IEEE80211_IOCTL_SETMLME]",
+			"ioctl[IEEE80211_IOCTL_GETCHANINFO]",
+			"ioctl[IEEE80211_IOCTL_SETOPTIE]",
+			"ioctl[IEEE80211_IOCTL_GETOPTIE]",
+			"ioctl[IEEE80211_IOCTL_ADDMAC]",
+			"ioctl[SIOCIWFIRSTPRIV+11]",
+			"ioctl[IEEE80211_IOCTL_DELMAC]",
+			"ioctl[IEEE80211_IOCTL_GETCHANLIST]",
+			"ioctl[IEEE80211_IOCTL_SETCHANLIST]",
+			"ioctl[SIOCIWFIRSTPRIV+15]",
+			"ioctl[IEEE80211_IOCTL_SETMODE]",
+			"ioctl[IEEE80211_IOCTL_GETMODE]",
+		};
+#else /* MADWIFI_NG */
+		int first = IEEE80211_IOCTL_SETPARAM;
+		int last = IEEE80211_IOCTL_CHANLIST;
 		static const char *opnames[] = {
 			"ioctl[IEEE80211_IOCTL_SETPARAM]",
 			"ioctl[IEEE80211_IOCTL_GETPARAM]",
@@ -118,15 +144,14 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 			"ioctl[SIOCIWFIRSTPRIV+17]",
 			"ioctl[IEEE80211_IOCTL_GETKEY]",
 		};
-		op -= SIOCIWFIRSTPRIV;
-		if (0 <= op && op < N(opnames))
-			perror(opnames[op]);
+#endif /* MADWIFI_NG */
+		if (first <= op && op <= last)
+			perror(opnames[op - first]);
 		else
 			perror("ioctl[unknown???]");
 		return -1;
 	}
 	return 0;
-#undef N
 }
 
 static int
@@ -339,7 +364,7 @@ madwifi_set_privacy(void *priv, int enabled)
 }
 
 static int
-madwifi_set_sta_authorized(void *priv, u8 *addr, int authorized)
+madwifi_set_sta_authorized(void *priv, const u8 *addr, int authorized)
 {
 	struct madwifi_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -360,7 +385,7 @@ madwifi_set_sta_authorized(void *priv, u8 *addr, int authorized)
 }
 
 static int
-madwifi_del_key(void *priv, unsigned char *addr, int key_idx)
+madwifi_del_key(void *priv, const u8 *addr, int key_idx)
 {
 	struct madwifi_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -383,8 +408,8 @@ madwifi_del_key(void *priv, unsigned char *addr, int key_idx)
 
 static int
 madwifi_set_key(void *priv, const char *alg,
-	     unsigned char *addr, int key_idx,
-	     u8 *key, size_t key_len)
+	     const u8 *addr, int key_idx,
+	     const u8 *key, size_t key_len)
 {
 	struct madwifi_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -435,7 +460,7 @@ madwifi_set_key(void *priv, const char *alg,
 
 
 static int
-madwifi_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
+madwifi_get_seqnum(void *priv, const u8 *addr, int idx, u8 *seq)
 {
 	struct madwifi_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -491,7 +516,7 @@ madwifi_flush(void *priv)
 
 static int
 madwifi_read_sta_driver_data(void *priv, struct hostap_sta_driver_data *data,
-					u8 *addr)
+			     const u8 *addr)
 {
 	struct madwifi_driver_data *drv = priv;
 
@@ -505,8 +530,13 @@ madwifi_read_sta_driver_data(void *priv, struct hostap_sta_driver_data *data,
 	 */
 	memset(&stats, 0, sizeof(stats));
 	memcpy(stats.is_u.macaddr, addr, IEEE80211_ADDR_LEN);
-	if (set80211priv(drv, IEEE80211_IOCTL_GETSTASTATS, &stats,
-			 sizeof(stats))) {
+	if (set80211priv(drv,
+#ifdef MADWIFI_NG
+			 IEEE80211_IOCTL_STA_STATS,
+#else /* MADWIFI_NG */
+			 IEEE80211_IOCTL_GETSTASTATS,
+#endif /* MADWIFI_NG */
+			 &stats, sizeof(stats))) {
 		if (memcmp(addr, drv->acct_mac, ETH_ALEN) == 0) {
 			memcpy(data, &drv->acct_data, sizeof(*data));
 			return 0;
@@ -567,9 +597,9 @@ madwifi_read_sta_driver_data(void *priv, struct hostap_sta_driver_data *data,
 
 
 static int
-madwifi_sta_clear_stats(void *priv, u8 *addr)
+madwifi_sta_clear_stats(void *priv, const u8 *addr)
 {
-#ifdef MADWIFI_BSD
+#if defined(MADWIFI_BSD) && defined(IEEE80211_MLME_CLEAR_STATS)
 	struct madwifi_driver_data *drv = priv;
 	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_mlme mlme;
@@ -581,9 +611,9 @@ madwifi_sta_clear_stats(void *priv, u8 *addr)
 	memcpy(mlme.im_macaddr, addr, IEEE80211_ADDR_LEN);
 	return set80211priv(priv, IEEE80211_IOCTL_SETMLME, &mlme,
 			    sizeof(mlme));
-#else /* MADWIFI_BSD */
+#else /* MADWIFI_BSD && IEEE80211_MLME_CLEAR_STATS */
 	return 0; /* FIX */
-#endif /* MADWIFI_BSD */
+#endif /* MADWIFI_BSD && IEEE80211_MLME_CLEAR_STATS */
 }
 
 
@@ -598,7 +628,7 @@ madwifi_set_opt_ie(void *priv, const u8 *ie, size_t ie_len)
 }
 
 static int
-madwifi_sta_deauth(void *priv, u8 *addr, int reason_code)
+madwifi_sta_deauth(void *priv, const u8 *addr, int reason_code)
 {
 	struct madwifi_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -615,7 +645,7 @@ madwifi_sta_deauth(void *priv, u8 *addr, int reason_code)
 }
 
 static int
-madwifi_sta_disassoc(void *priv, u8 *addr, int reason_code)
+madwifi_sta_disassoc(void *priv, const u8 *addr, int reason_code)
 {
 	struct madwifi_driver_data *drv = priv;
 	hostapd *hapd = drv->hapd;
@@ -642,9 +672,9 @@ madwifi_del_sta(struct madwifi_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 	sta = ap_get_sta(hapd, addr);
 	if (sta != NULL) {
 		sta->flags &= ~WLAN_STA_ASSOC;
-		wpa_sm_event(hapd, sta, WPA_DISASSOC);
+		wpa_auth_sm_event(sta->wpa_sm, WPA_DISASSOC);
 		sta->acct_terminate_cause = RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-		ieee802_1x_set_port_enabled(hapd, sta, 0);
+		ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 		ap_free_sta(hapd, sta);
 	}
 	return 0;
@@ -672,21 +702,18 @@ madwifi_process_wpa_ie(struct madwifi_driver_data *drv, struct sta_info *sta)
 		return -1;		/* XXX not right */
 	}
 	ielen += 2;
-	res = wpa_validate_wpa_ie(hapd, sta, ie.wpa_ie, ielen,
-			ie.wpa_ie[0] == WLAN_EID_RSN ?
-			    HOSTAPD_WPA_VERSION_WPA2 : HOSTAPD_WPA_VERSION_WPA);
+	if (sta->wpa_sm == NULL)
+		sta->wpa_sm = wpa_auth_sta_init(hapd->wpa_auth, sta->addr);
+	if (sta->wpa_sm == NULL) {
+		printf("Failed to initialize WPA state machine\n");
+		return -1;
+	}
+	res = wpa_validate_wpa_ie(hapd->wpa_auth, sta->wpa_sm,
+				  ie.wpa_ie, ielen);
 	if (res != WPA_IE_OK) {
 		printf("WPA/RSN information element rejected? (res %u)\n", res);
 		return -1;
 	}
-	free(sta->wpa_ie);
-	sta->wpa_ie = malloc(ielen);
-	if (sta->wpa_ie == NULL) {
-		printf("No memory to save WPA/RSN information element!\n");
-		return -1;
-	}
-	memcpy(sta->wpa_ie, ie.wpa_ie, ielen);
-	sta->wpa_ie_len = ielen;
 	return 0;
 }
 
@@ -719,10 +746,6 @@ madwifi_new_sta(struct madwifi_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 	if (hapd->conf->wpa) {
 		if (madwifi_process_wpa_ie(drv, sta))
 			return -1;
-	} else {
-		free(sta->wpa_ie);
-		sta->wpa_ie = NULL;
-		sta->wpa_ie_len = 0;
 	}
 
 	/*
@@ -731,7 +754,7 @@ madwifi_new_sta(struct madwifi_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 	 */
 	new_assoc = (sta->flags & WLAN_STA_ASSOC) == 0;
 	sta->flags |= WLAN_STA_ASSOC;
-	wpa_sm_event(hapd, sta, WPA_ASSOC);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_ASSOC);
 	hostapd_new_assoc_sta(hapd, sta, !new_assoc);
 	ieee802_1x_notify_port_enabled(sta->eapol_sm, 1);
 	return 0;
@@ -826,10 +849,10 @@ madwifi_wireless_event_wireless(struct madwifi_driver_data *drv,
 
 		switch (iwe->cmd) {
 		case IWEVEXPIRED:
-			madwifi_del_sta(drv, iwe->u.addr.sa_data);
+			madwifi_del_sta(drv, (u8 *) iwe->u.addr.sa_data);
 			break;
 		case IWEVREGISTERED:
-			madwifi_new_sta(drv, iwe->u.addr.sa_data);
+			madwifi_new_sta(drv, (u8 *) iwe->u.addr.sa_data);
 			break;
 		case IWEVCUSTOM:
 			if (custom + iwe->u.data.length > end)
@@ -1029,10 +1052,10 @@ madwifi_wireless_event_deinit(void *priv)
 
 
 static int
-madwifi_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len, int encrypt)
+madwifi_send_eapol(void *priv, const u8 *addr, const u8 *data, size_t data_len,
+		   int encrypt)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
 	unsigned char buf[3000];
 	unsigned char *bp = buf;
 	struct l2_ethhdr *eth;
@@ -1060,8 +1083,7 @@ madwifi_send_eapol(void *priv, u8 *addr, u8 *data, size_t data_len, int encrypt)
 	eth->h_proto = htons(ETH_P_EAPOL);
 	memcpy(eth+1, data, data_len);
 
-	if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MSGDUMPS))
-		hostapd_hexdump("TX EAPOL", bp, len);
+	wpa_hexdump(MSG_MSGDUMP, "TX EAPOL", bp, len);
 
 	status = l2_packet_send(drv->sock_xmit, addr, ETH_P_EAPOL, bp, len);
 
@@ -1131,7 +1153,7 @@ madwifi_init(struct hostapd_data *hapd)
 			hapd->conf->bridge);
 		drv->sock_recv = l2_packet_init(hapd->conf->bridge, NULL,
 						ETH_P_EAPOL, handle_read, drv,
-						0);
+						1);
 		if (drv->sock_recv == NULL)
 			goto bad;
 	} else
@@ -1181,7 +1203,7 @@ madwifi_deinit(void *priv)
 }
 
 static int
-madwifi_set_ssid(void *priv, u8 *buf, int len)
+madwifi_set_ssid(void *priv, const u8 *buf, int len)
 {
 	struct madwifi_driver_data *drv = priv;
 	struct iwreq iwr;
@@ -1221,6 +1243,14 @@ madwifi_get_ssid(void *priv, u8 *buf, int len)
 	return ret;
 }
 
+static int
+madwifi_set_countermeasures(void *priv, int enabled)
+{
+	struct madwifi_driver_data *drv = priv;
+	wpa_printf(MSG_DEBUG, "%s: enabled=%d", __FUNCTION__, enabled);
+	return set80211param(drv, IEEE80211_PARAM_COUNTERMEASURES, enabled);
+}
+
 static const struct driver_ops madwifi_driver_ops = {
 	.name			= "madwifi",
 	.init			= madwifi_init,
@@ -1240,6 +1270,7 @@ static const struct driver_ops madwifi_driver_ops = {
 	.sta_deauth		= madwifi_sta_deauth,
 	.set_ssid		= madwifi_set_ssid,
 	.get_ssid		= madwifi_get_ssid,
+	.set_countermeasures	= madwifi_set_countermeasures,
 	.sta_clear_stats        = madwifi_sta_clear_stats,
 };
 
