@@ -13,20 +13,9 @@
  * See README and COPYING for more details.
  */
 
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <netinet/in.h>
-
-/* for select */
 #include <sys/select.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <unistd.h>
 
 #ifdef USE_KERNEL_HEADERS
 #include <asm/types.h>
@@ -138,8 +127,8 @@ static int prism54_waitpim(void *priv, unsigned long oid, void *buf, int len,
 
 
 /* send an eapol packet */
-static int prism54_send_eapol(void *priv, u8 *addr,
-			      u8 *data, size_t data_len, int encrypt)
+static int prism54_send_eapol(void *priv, const u8 *addr,
+			      const u8 *data, size_t data_len, int encrypt)
 {
 	struct prism54_driver_data *drv = priv;
 	ieee802_3_hdr *hdr;
@@ -177,7 +166,8 @@ static int prism54_send_eapol(void *priv, u8 *addr,
 
 
 /* open data channel(auth-1) or eapol only(unauth-0) */
-static int prism54_set_sta_authorized(void *priv, u8 *addr, int authorized)
+static int prism54_set_sta_authorized(void *priv, const u8 *addr,
+				      int authorized)
 {
 	struct prism54_driver_data *drv = priv;
 	pimdev_hdr *hdr;
@@ -202,8 +192,8 @@ static int prism54_set_sta_authorized(void *priv, u8 *addr, int authorized)
 
 
 /* set per station key */
-static int prism54_set_encryption(void *priv, const char *alg, u8 *addr,
-				  int idx, u8 *key, size_t key_len)
+static int prism54_set_encryption(void *priv, const char *alg, const u8 *addr,
+				  int idx, const u8 *key, size_t key_len)
 {
 	struct prism54_driver_data *drv = priv;
 	pimdev_hdr *hdr;
@@ -259,7 +249,7 @@ static int prism54_set_encryption(void *priv, const char *alg, u8 *addr,
 
 
 /* get TKIP station sequence counter, prism54 is only 6 bytes */
-static int prism54_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
+static int prism54_get_seqnum(void *priv, const u8 *addr, int idx, u8 *seq)
 {
 	struct prism54_driver_data *drv = priv;
 	struct obj_stasc *stasc;
@@ -369,7 +359,7 @@ static int prism54_set_privacy_invoked(void *priv, int flag)
 }
 
  
-static int prism54_ioctl_setiwessid(void *priv, u8 *buf, int len)
+static int prism54_ioctl_setiwessid(void *priv, const u8 *buf, int len)
 {
 #if 0
 	struct prism54_driver_data *drv = priv;
@@ -446,7 +436,7 @@ static int prism54_flush(void *priv)
 }
 
 
-static int prism54_sta_deauth(void *priv, u8 *addr, int reason)
+static int prism54_sta_deauth(void *priv, const u8 *addr, int reason)
 {
 	struct prism54_driver_data *drv = priv;
 	pimdev_hdr *hdr;
@@ -471,7 +461,7 @@ static int prism54_sta_deauth(void *priv, u8 *addr, int reason)
 }
 
 
-static int prism54_sta_disassoc(void *priv, u8 *addr, int reason)
+static int prism54_sta_disassoc(void *priv, const u8 *addr, int reason)
 {
 	struct prism54_driver_data *drv = priv;
         pimdev_hdr *hdr;
@@ -496,7 +486,7 @@ static int prism54_sta_disassoc(void *priv, u8 *addr, int reason)
 }
 
 
-static int prism54_get_inact_sec(void *priv, u8 *addr)
+static int prism54_get_inact_sec(void *priv, const u8 *addr)
 {
 	struct prism54_driver_data *drv = priv;
 	pimdev_hdr *hdr;
@@ -602,9 +592,9 @@ static void prism54_handle_deauth(struct prism54_driver_data *drv,
 		return;
 	}
 	sta->flags &= ~(WLAN_STA_AUTH | WLAN_STA_ASSOC);
-	wpa_sm_event(drv->hapd, sta, WPA_DEAUTH);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_DEAUTH);
 	sta->acct_terminate_cause = RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-	ieee802_1x_set_port_enabled(drv->hapd, sta, 0);
+	ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 	ap_free_sta(drv->hapd, sta);
 }
 
@@ -623,9 +613,9 @@ static void prism54_handle_disassoc(struct prism54_driver_data *drv,
 		return;
 	}
 	sta->flags &= ~WLAN_STA_ASSOC;
-	wpa_sm_event(drv->hapd, sta, WPA_DISASSOC);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_DISASSOC);
 	sta->acct_terminate_cause = RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-	ieee802_1x_set_port_enabled(drv->hapd, sta, 0);
+	ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 	accounting_sta_stop(drv->hapd, sta);
 	ieee802_1x_free_station(sta);
 }
@@ -662,7 +652,7 @@ static void prism54_handle_auth(struct prism54_driver_data *drv,
 		
 		ieee802_1x_notify_pre_auth(sta->eapol_sm, 0);
 		sta->flags |= WLAN_STA_AUTH;
-		wpa_sm_event(drv->hapd, sta, WPA_AUTH);
+		wpa_auth_sm_event(sta->wpa_sm, WPA_AUTH);
 		mlme->code = 0;
 		mlme->state=htons(DOT11_STATE_AUTH);
 		hdr->op = htonl(PIMOP_SET);
@@ -760,29 +750,30 @@ static void prism54_handle_assoc(struct prism54_driver_data *drv,
 			int res;
 			wpa_ie -= 2;
 			wpa_ie_len += 2;
-			res = wpa_validate_wpa_ie(drv->hapd, sta, wpa_ie,
-						  wpa_ie_len, elems.rsn_ie ?
-						  HOSTAPD_WPA_VERSION_WPA2 :
-						  HOSTAPD_WPA_VERSION_WPA);
+			if (sta->wpa_sm == NULL)
+				sta->wpa_sm = wpa_auth_sta_init(
+					drv->hapd->wpa_auth, sta->addr);
+			if (sta->wpa_sm == NULL) {
+				printf("Failed to initialize WPA state "
+				       "machine\n");
+				resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
+				goto fail;
+			}
+			res = wpa_validate_wpa_ie(drv->hapd->wpa_auth,
+						  sta->wpa_sm,
+						  wpa_ie, wpa_ie_len);
 			if (res == WPA_INVALID_GROUP)
 				resp = WLAN_STATUS_GROUP_CIPHER_NOT_VALID;
 			else if (res == WPA_INVALID_PAIRWISE)
 				resp = WLAN_STATUS_PAIRWISE_CIPHER_NOT_VALID;
 			else if (res == WPA_INVALID_AKMP)
 				resp = WLAN_STATUS_AKMP_NOT_VALID;
+			else if (res == WPA_ALLOC_FAIL)
+				resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
 			else if (res != WPA_IE_OK)
 				resp = WLAN_STATUS_INVALID_IE;
 			if (resp != WLAN_STATUS_SUCCESS)
 				goto fail;
-			if (sta->wpa_ie != NULL)
-				free(sta->wpa_ie);
-			sta->wpa_ie = malloc(wpa_ie_len);
-			if (sta->wpa_ie == NULL) {
-				resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
-				goto fail;
-			}
-			sta->wpa_ie_len = wpa_ie_len;
-			memcpy(sta->wpa_ie, wpa_ie, wpa_ie_len);
 		}
 		hdr->oid = (hdr->oid == htonl(DOT11_OID_ASSOCIATEEX)) ?
 			htonl(DOT11_OID_ASSOCIATEEX) :
@@ -806,7 +797,7 @@ static void prism54_handle_assoc(struct prism54_driver_data *drv,
 		}
 		new_assoc = (sta->flags & WLAN_STA_ASSOC) == 0;
 		sta->flags |= WLAN_STA_ASSOC;
-		wpa_sm_event(drv->hapd, sta, WPA_ASSOC);
+		wpa_auth_sm_event(sta->wpa_sm, WPA_ASSOC);
 		hostapd_new_assoc_sta(drv->hapd, sta, !new_assoc);
 		ieee802_1x_notify_port_enabled(sta->eapol_sm, 1);
 		sta->timeout_next = STA_NULLFUNC;
