@@ -13,18 +13,9 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <netinet/in.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <sys/un.h>
-#include <sys/uio.h>
 
 #include "hostapd.h"
 #include "driver.h"
@@ -37,6 +28,7 @@
 #include "accounting.h"
 #include "radius.h"
 #include "l2_packet.h"
+#include "hostap_common.h"
 
 
 struct test_client_socket {
@@ -75,7 +67,7 @@ test_driver_get_cli(struct test_driver_data *drv, struct sockaddr_un *from,
 }
 
 
-static int test_driver_send_eapol(void *priv, u8 *addr, u8 *data,
+static int test_driver_send_eapol(void *priv, const u8 *addr, const u8 *data,
 				  size_t data_len, int encrypt)
 {
 	struct test_driver_data *drv = priv;
@@ -105,7 +97,7 @@ static int test_driver_send_eapol(void *priv, u8 *addr, u8 *data,
 	io[0].iov_len = 6;
 	io[1].iov_base = &eth;
 	io[1].iov_len = sizeof(eth);
-	io[2].iov_base = data;
+	io[2].iov_base = (u8 *) data;
 	io[2].iov_len = data_len;
 
 	memset(&msg, 0, sizeof(msg));
@@ -171,35 +163,28 @@ static int test_driver_new_sta(struct test_driver_data *drv, const u8 *addr,
 			printf("test_driver: no IE from STA\n");
 			return -1;
 		}
-		res = wpa_validate_wpa_ie(hapd, sta, ie, ielen,
-					  ie[0] == WLAN_EID_RSN ?
-					  HOSTAPD_WPA_VERSION_WPA2 :
-					  HOSTAPD_WPA_VERSION_WPA);
+		if (sta->wpa_sm == NULL)
+			sta->wpa_sm = wpa_auth_sta_init(hapd->wpa_auth,
+							sta->addr);
+		if (sta->wpa_sm == NULL) {
+			printf("test_driver: Failed to initialize WPA state "
+			       "machine\n");
+			return -1;
+		}
+		res = wpa_validate_wpa_ie(hapd->wpa_auth, sta->wpa_sm,
+					  ie, ielen);
 		if (res != WPA_IE_OK) {
 			printf("WPA/RSN information element rejected? "
 			       "(res %u)\n", res);
 			return -1;
 		}
-		free(sta->wpa_ie);
-		sta->wpa_ie = malloc(ielen);
-		if (sta->wpa_ie == NULL)
-			return -1;
-		memcpy(sta->wpa_ie, ie, ielen);
-		sta->wpa_ie_len = ielen;
-	} else {
-		free(sta->wpa_ie);
-		sta->wpa_ie = NULL;
-		sta->wpa_ie_len = 0;
 	}
 
 	new_assoc = (sta->flags & WLAN_STA_ASSOC) == 0;
 	sta->flags |= WLAN_STA_ASSOC;
-	wpa_sm_event(hapd, sta, WPA_ASSOC);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_ASSOC);
 
-	if (new_assoc)
-		hostapd_new_assoc_sta(hapd, sta);
-	else
-		wpa_sm_event(hapd, sta, WPA_REAUTH);
+	hostapd_new_assoc_sta(hapd, sta, !new_assoc);
 
 	ieee802_1x_notify_port_enabled(sta->eapol_sm, 1);
 
@@ -279,10 +264,10 @@ static void test_driver_disassoc(struct test_driver_data *drv,
 	sta = ap_get_sta(drv->hapd, cli->addr);
 	if (sta != NULL) {
 		sta->flags &= ~WLAN_STA_ASSOC;
-		wpa_sm_event(drv->hapd, sta, WPA_DISASSOC);
+		wpa_auth_sm_event(sta->wpa_sm, WPA_DISASSOC);
 		sta->acct_terminate_cause =
 			RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-		ieee802_1x_set_port_enabled(drv->hapd, sta, 0);
+		ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 		ap_free_sta(drv->hapd, sta);
 	}
 }
@@ -359,6 +344,52 @@ static int test_driver_set_generic_elem(void *priv,
 }
 
 
+static int test_driver_sta_deauth(void *priv, const u8 *addr, int reason)
+{
+	struct test_driver_data *drv = priv;
+	struct test_client_socket *cli;
+
+	if (drv->test_socket < 0)
+		return -1;
+
+	cli = drv->cli;
+	while (cli) {
+		if (memcmp(cli->addr, addr, ETH_ALEN) == 0)
+			break;
+		cli = cli->next;
+	}
+
+	if (!cli)
+		return -1;
+
+	return sendto(drv->test_socket, "DEAUTH", 6, 0,
+		      (struct sockaddr *) &cli->un, cli->unlen);
+}
+
+
+static int test_driver_sta_disassoc(void *priv, const u8 *addr, int reason)
+{
+	struct test_driver_data *drv = priv;
+	struct test_client_socket *cli;
+
+	if (drv->test_socket < 0)
+		return -1;
+
+	cli = drv->cli;
+	while (cli) {
+		if (memcmp(cli->addr, addr, ETH_ALEN) == 0)
+			break;
+		cli = cli->next;
+	}
+
+	if (!cli)
+		return -1;
+
+	return sendto(drv->test_socket, "DISASSOC", 8, 0,
+		      (struct sockaddr *) &cli->un, cli->unlen);
+}
+
+
 static int test_driver_init(struct hostapd_data *hapd)
 {
 	struct test_driver_data *drv;
@@ -447,6 +478,8 @@ static const struct driver_ops test_driver_ops = {
 	.deinit = test_driver_deinit,
 	.send_eapol = test_driver_send_eapol,
 	.set_generic_elem = test_driver_set_generic_elem,
+	.sta_deauth = test_driver_sta_deauth,
+	.sta_disassoc = test_driver_sta_disassoc,
 };
 
 
