@@ -14,16 +14,8 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <errno.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <netinet/in.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 
 #ifdef USE_KERNEL_HEADERS
 #include <asm/types.h>
@@ -51,6 +43,7 @@ struct wired_driver_data {
 
 	int sock; /* raw packet socket for driver access */
 	int dhcp_sock; /* socket for dhcp packets */
+	int use_pae_group_addr;
 };
 
 static const struct driver_ops wired_driver_ops;
@@ -305,11 +298,11 @@ static int wired_init_sockets(struct wired_driver_data *drv)
 }
 
 
-static int wired_send_eapol(void *priv, u8 *addr,
-			    u8 *data, size_t data_len, int encrypt)
+static int wired_send_eapol(void *priv, const u8 *addr,
+			    const u8 *data, size_t data_len, int encrypt)
 {
 	struct wired_driver_data *drv = priv;
-
+	u8 pae_group_addr[ETH_ALEN] = WIRED_EAPOL_MULTICAST_GROUP;
 	struct ieee8023_hdr *hdr;
 	size_t len;
 	u8 *pos;
@@ -324,7 +317,8 @@ static int wired_send_eapol(void *priv, u8 *addr,
 	}
 
 	memset(hdr, 0, len);
-	memcpy(hdr->dest, addr, ETH_ALEN);
+	memcpy(hdr->dest, drv->use_pae_group_addr ? pae_group_addr : addr,
+	       ETH_ALEN);
 	memcpy(hdr->src, drv->hapd->own_addr, ETH_ALEN);
 	hdr->ethertype = htons(ETH_P_PAE);
 
@@ -357,6 +351,7 @@ static int wired_driver_init(struct hostapd_data *hapd)
 	memset(drv, 0, sizeof(*drv));
 	drv->ops = wired_driver_ops;
 	drv->hapd = hapd;
+	drv->use_pae_group_addr = hapd->conf->use_pae_group_addr;
 
 	if (wired_init_sockets(drv))
 		return -1;
