@@ -1,5 +1,5 @@
 /*
- * hostapd / EAP Standalone Authenticator state machine
+ * hostapd / EAP Standalone Authenticator state machine (RFC 4137)
  * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -12,17 +12,15 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <string.h>
-#include <sys/socket.h>
+#include "includes.h"
 
 #include "hostapd.h"
-#include "eloop.h"
 #include "sta_info.h"
 #include "eap_i.h"
+#include "state_machine.h"
+
+#define STATE_MACHINE_DATA struct eap_sm
+#define STATE_MACHINE_DEBUG_PREFIX "EAP"
 
 #define EAP_MAX_AUTH_ROUNDS 50
 
@@ -51,6 +49,9 @@ extern const struct eap_method eap_method_ttls;
 #ifdef EAP_SIM
 extern const struct eap_method eap_method_sim;
 #endif /* EAP_SIM */
+#ifdef EAP_AKA
+extern const struct eap_method eap_method_aka;
+#endif /* EAP_AKA */
 #ifdef EAP_PAX
 extern const struct eap_method eap_method_pax;
 #endif /* EAP_PAX */
@@ -85,6 +86,9 @@ static const struct eap_method *eap_methods[] =
 #ifdef EAP_SIM
 	&eap_method_sim,
 #endif /* EAP_SIM */
+#ifdef EAP_AKA
+	&eap_method_aka,
+#endif /* EAP_AKA */
 #ifdef EAP_PAX
 	&eap_method_pax,
 #endif /* EAP_PAX */
@@ -108,7 +112,7 @@ const struct eap_method * eap_sm_get_eap_methods(int method)
 static void eap_user_free(struct eap_user *user);
 
 
-/* EAP state machines are described in draft-ietf-eap-statemachine-05.txt */
+/* EAP state machines are described in RFC 4137 */
 
 static int eap_sm_calculateTimeout(struct eap_sm *sm, int retransCount,
 				   int eapSRTT, int eapRTTVAR,
@@ -123,29 +127,6 @@ static int eap_sm_Policy_getDecision(struct eap_sm *sm);
 static Boolean eap_sm_Policy_doPickUp(struct eap_sm *sm, EapType method);
 
 
-/* Definitions for clarifying state machine implementation */
-#define SM_STATE(machine, state) \
-static void sm_ ## machine ## _ ## state ## _Enter(struct eap_sm *sm, \
-	int global)
-
-#define SM_ENTRY(machine, state) \
-if (!global || sm->machine ## _state != machine ## _ ## state) { \
-	sm->changed = TRUE; \
-	wpa_printf(MSG_DEBUG, "EAP: " #machine " entering state " #state); \
-} \
-sm->machine ## _state = machine ## _ ## state;
-
-#define SM_ENTER(machine, state) \
-sm_ ## machine ## _ ## state ## _Enter(sm, 0)
-#define SM_ENTER_GLOBAL(machine, state) \
-sm_ ## machine ## _ ## state ## _Enter(sm, 1)
-
-#define SM_STEP(machine) \
-static void sm_ ## machine ## _Step(struct eap_sm *sm)
-
-#define SM_STEP_RUN(machine) sm_ ## machine ## _Step(sm)
-
-
 static Boolean eapol_get_bool(struct eap_sm *sm, enum eapol_bool_var var)
 {
 	return sm->eapol_cb->get_bool(sm->eapol_ctx, var);
@@ -228,9 +209,11 @@ SM_STATE(EAP, INITIALIZE)
 	/* eapKeyAvailable = FALSE */
 	eapol_set_bool(sm, EAPOL_eapRestart, FALSE);
 
-	/* This is not defined in draft-ietf-eap-statemachine-05.txt, but
-	 * method state needs to be reseted here so that it does not remain in
-	 * success state when re-authentication starts. */
+	/*
+	 * This is not defined in RFC 4137, but method state needs to be
+	 * reseted here so that it does not remain in success state when
+	 * re-authentication starts.
+	 */
 	if (sm->m && sm->eap_method_priv) {
 		sm->m->reset(sm, sm->eap_method_priv);
 		sm->eap_method_priv = NULL;
@@ -247,6 +230,7 @@ SM_STATE(EAP, INITIALIZE)
 		}
 	}
 	sm->num_rounds = 0;
+	sm->method_pending = METHOD_PENDING_NONE;
 }
 
 
@@ -593,9 +577,25 @@ SM_STEP(EAP)
 		SM_ENTER(EAP, SEND_REQUEST);
 		break;
 	case EAP_METHOD_RESPONSE:
+		/*
+		 * Note: Mechanism to allow EAP methods to wait while going
+		 * through pending processing is an extension to RFC 4137
+		 * which only defines the transits to SELECT_ACTION and
+		 * METHOD_REQUEST from this METHOD_RESPONSE state.
+		 */
 		if (sm->methodState == METHOD_END)
 			SM_ENTER(EAP, SELECT_ACTION);
-		else
+		else if (sm->method_pending == METHOD_PENDING_WAIT) {
+			wpa_printf(MSG_DEBUG, "EAP: Method has pending "
+				   "processing - wait before proceeding to "
+				   "METHOD_REQUEST state");
+		} else if (sm->method_pending == METHOD_PENDING_CONT) {
+			wpa_printf(MSG_DEBUG, "EAP: Method has completed "
+				   "pending processing - reprocess pending "
+				   "EAP message");
+			sm->method_pending = METHOD_PENDING_NONE;
+			SM_ENTER(EAP, METHOD_RESPONSE);
+		} else
 			SM_ENTER(EAP, METHOD_REQUEST);
 		break;
 	case EAP_PROPOSE_METHOD:
@@ -942,3 +942,21 @@ void eap_sm_notify_cached(struct eap_sm *sm)
 
 	sm->EAP_state = EAP_SUCCESS;
 }
+
+
+void eap_sm_pending_cb(struct eap_sm *sm)
+{
+	if (sm == NULL)
+		return;
+	wpa_printf(MSG_DEBUG, "EAP: Callback for pending request received");
+	if (sm->method_pending == METHOD_PENDING_WAIT)
+		sm->method_pending = METHOD_PENDING_CONT;
+}
+
+
+int eap_sm_method_pending(struct eap_sm *sm)
+{
+	if (sm == NULL)
+		return 0;
+	return sm->method_pending == METHOD_PENDING_WAIT;
+}
