@@ -11,6 +11,8 @@ struct eap_user {
 	u8 methods[EAP_MAX_METHODS];
 	u8 *password;
 	size_t password_len;
+	int password_hash; /* whether password is hashed with
+			    * nt_password_hash() */
 	int phase2;
 	int force_version;
 };
@@ -50,6 +52,8 @@ u8 eap_get_type(const char *name);
 void eap_set_eapRespData(struct eap_sm *sm, const u8 *eapRespData,
 			 size_t eapRespDataLen);
 void eap_sm_notify_cached(struct eap_sm *sm);
+void eap_sm_pending_cb(struct eap_sm *sm);
+int eap_sm_method_pending(struct eap_sm *sm);
 
 #else /* EAP_SERVER */
 
@@ -84,6 +88,15 @@ static inline void eap_sm_notify_cached(struct eap_sm *sm)
 {
 }
 
+static inline void eap_sm_pending_cb(struct eap_sm *sm)
+{
+}
+
+int eap_sm_method_pending(struct eap_sm *sm)
+{
+	return 0;
+}
+
 #endif /* EAP_SERVER */
 
 #endif /* EAP_H */
