@@ -102,6 +102,9 @@ struct eap_sm {
 	Boolean update_user;
 
 	int num_rounds;
+	enum {
+		METHOD_PENDING_NONE, METHOD_PENDING_WAIT, METHOD_PENDING_CONT
+	} method_pending;
 };
 
 const struct eap_method * eap_sm_get_eap_methods(int method);
