@@ -12,10 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <netinet/in.h>
+#include "includes.h"
 
 #include "hostapd.h"
 #include "common.h"
@@ -23,6 +20,7 @@
 #include "eap_tls_common.h"
 #include "ms_funcs.h"
 #include "md5.h"
+#include "crypto.h"
 #include "tls.h"
 #include "eap_ttls.h"
 
@@ -540,9 +538,9 @@ static void eap_ttls_process_phase2_pap(struct eap_sm *sm,
 {
 	/* TODO: add support for verifying that the user entry accepts
 	 * EAP-TTLS/PAP. */
-	if (!sm->user || !sm->user->password) {
-		wpa_printf(MSG_DEBUG, "EAP-TTLS/PAP: No user password "
-			   "configured");
+	if (!sm->user || !sm->user->password || sm->user->password_hash) {
+		wpa_printf(MSG_DEBUG, "EAP-TTLS/PAP: No plaintext user "
+			   "password configured");
 		eap_ttls_state(data, FAILURE);
 		return;
 	}
@@ -567,8 +565,9 @@ static void eap_ttls_process_phase2_chap(struct eap_sm *sm,
 					 const u8 *password,
 					 size_t password_len)
 {
-	MD5_CTX context;
 	u8 *chal, hash[MD5_MAC_LEN];
+	const u8 *addr[3];
+	size_t len[3];
 
 	if (challenge == NULL || password == NULL ||
 	    challenge_len != EAP_TTLS_CHAP_CHALLENGE_LEN ||
@@ -583,9 +582,9 @@ static void eap_ttls_process_phase2_chap(struct eap_sm *sm,
 
 	/* TODO: add support for verifying that the user entry accepts
 	 * EAP-TTLS/CHAP. */
-	if (!sm->user || !sm->user->password) {
-		wpa_printf(MSG_DEBUG, "EAP-TTLS/CHAP: No user password "
-			   "configured");
+	if (!sm->user || !sm->user->password || sm->user->password_hash) {
+		wpa_printf(MSG_DEBUG, "EAP-TTLS/CHAP: No plaintext user "
+			   "password configured");
 		eap_ttls_state(data, FAILURE);
 		return;
 	}
@@ -609,11 +608,13 @@ static void eap_ttls_process_phase2_chap(struct eap_sm *sm,
 	free(chal);
 
 	/* MD5(Ident + Password + Challenge) */
-	MD5Init(&context);
-	MD5Update(&context, password, 1);
-	MD5Update(&context, sm->user->password, sm->user->password_len);
-	MD5Update(&context, challenge, challenge_len);
-	MD5Final(hash, &context);
+	addr[0] = password;
+	len[0] = 1;
+	addr[1] = sm->user->password;
+	len[1] = sm->user->password_len;
+	addr[2] = challenge;
+	len[2] = challenge_len;
+	md5_vector(3, addr, len, hash);
 
 	if (memcmp(hash, password + 1, EAP_TTLS_CHAP_PASSWORD_LEN) == 0) {
 		wpa_printf(MSG_DEBUG, "EAP-TTLS/CHAP: Correct user password");
@@ -670,8 +671,11 @@ static void eap_ttls_process_phase2_mschap(struct eap_sm *sm,
 	}
 	free(chal);
 
-	nt_challenge_response(challenge, sm->user->password,
-			      sm->user->password_len, nt_response);
+	if (sm->user->password_hash)
+		challenge_response(challenge, sm->user->password, nt_response);
+	else
+		nt_challenge_response(challenge, sm->user->password,
+				      sm->user->password_len, nt_response);
 
 	if (memcmp(nt_response, response + 2 + 24, 24) == 0) {
 		wpa_printf(MSG_DEBUG, "EAP-TTLS/MSCHAP: Correct response");
@@ -760,10 +764,18 @@ static void eap_ttls_process_phase2_mschapv2(struct eap_sm *sm,
 	wpa_hexdump(MSG_MSGDUMP, "EAP-TTLS/MSCHAPV2: peer_challenge",
 		    peer_challenge, EAP_TTLS_MSCHAPV2_CHALLENGE_LEN);
 
-	generate_nt_response(auth_challenge, peer_challenge,
-			     username, username_len,
-			     sm->user->password, sm->user->password_len,
-			     nt_response);
+	if (sm->user->password_hash) {
+		generate_nt_response_pwhash(auth_challenge, peer_challenge,
+					    username, username_len,
+					    sm->user->password,
+					    nt_response);
+	} else {
+		generate_nt_response(auth_challenge, peer_challenge,
+				     username, username_len,
+				     sm->user->password,
+				     sm->user->password_len,
+				     nt_response);
+	}
 
 	rx_resp = response + 2 + EAP_TTLS_MSCHAPV2_CHALLENGE_LEN + 8;
 	if (memcmp(nt_response, rx_resp, 24) == 0) {
@@ -771,14 +783,19 @@ static void eap_ttls_process_phase2_mschapv2(struct eap_sm *sm,
 			   "NT-Response");
 		data->mschapv2_resp_ok = 1;
 
-		generate_authenticator_response(sm->user->password,
-						sm->user->password_len,
-						peer_challenge,
-						auth_challenge,
-						username, username_len,
-						nt_response,
-						data->mschapv2_auth_response);
-
+		if (sm->user->password_hash) {
+			generate_authenticator_response_pwhash(
+				sm->user->password,
+				peer_challenge, auth_challenge,
+				username, username_len, nt_response,
+				data->mschapv2_auth_response);
+		} else {
+			generate_authenticator_response(
+				sm->user->password, sm->user->password_len,
+				peer_challenge, auth_challenge,
+				username, username_len, nt_response,
+				data->mschapv2_auth_response);
+		}
 	} else {
 		wpa_printf(MSG_DEBUG, "EAP-TTLS/MSCHAPV2: Invalid "
 			   "NT-Response");
