@@ -3,7 +3,7 @@
 
 #include "defs.h"
 
-/* IEEE Std 802.1X-REV-d11, Ch. 8.2 */
+/* IEEE Std 802.1X-2004, Ch. 8.2 */
 
 typedef enum { ForceUnauthorized = 1, ForceAuthorized = 3, Auto = 2 }
 	PortTypes;
@@ -11,103 +11,6 @@ typedef enum { Unauthorized = 2, Authorized = 1 } PortState;
 typedef enum { Both = 0, In = 1 } ControlledDirection;
 typedef unsigned int Counter;
 
-
-/* Authenticator PAE state machine */
-struct eapol_auth_pae_sm {
-	/* variables */
-	Boolean eapolLogoff;
-	Boolean eapolStart;
-	Boolean eapRestart;
-	PortTypes portMode;
-	unsigned int reAuthCount;
-
-	/* constants */
-	unsigned int quietPeriod; /* default 60; 0..65535 */
-#define AUTH_PAE_DEFAULT_quietPeriod 60
-	unsigned int reAuthMax; /* default 2 */
-#define AUTH_PAE_DEFAULT_reAuthMax 2
-
-	/* counters */
-	Counter authEntersConnecting;
-	Counter authEapLogoffsWhileConnecting;
-	Counter authEntersAuthenticating;
-	Counter authAuthSuccessesWhileAuthenticating;
-	Counter authAuthTimeoutsWhileAuthenticating;
-	Counter authAuthFailWhileAuthenticating;
-	Counter authAuthEapStartsWhileAuthenticating;
-	Counter authAuthEapLogoffWhileAuthenticating;
-	Counter authAuthReauthsWhileAuthenticated;
-	Counter authAuthEapStartsWhileAuthenticated;
-	Counter authAuthEapLogoffWhileAuthenticated;
-
-	enum { AUTH_PAE_INITIALIZE, AUTH_PAE_DISCONNECTED, AUTH_PAE_CONNECTING,
-	       AUTH_PAE_AUTHENTICATING, AUTH_PAE_AUTHENTICATED,
-	       AUTH_PAE_ABORTING, AUTH_PAE_HELD, AUTH_PAE_FORCE_AUTH,
-	       AUTH_PAE_FORCE_UNAUTH, AUTH_PAE_RESTART } state;
-};
-
-
-/* Backend Authentication state machine */
-struct eapol_backend_auth_sm {
-	/* variables */
-	Boolean eapNoReq;
-	Boolean eapReq;
-	Boolean eapResp;
-
-	/* constants */
-	unsigned int serverTimeout; /* default 30; 1..X */
-#define BE_AUTH_DEFAULT_serverTimeout 30
-
-	/* counters */
-	Counter backendResponses;
-	Counter backendAccessChallenges;
-	Counter backendOtherRequestsToSupplicant;
-	Counter backendAuthSuccesses;
-	Counter backendAuthFails;
-
-	enum { BE_AUTH_REQUEST, BE_AUTH_RESPONSE, BE_AUTH_SUCCESS,
-	       BE_AUTH_FAIL, BE_AUTH_TIMEOUT, BE_AUTH_IDLE, BE_AUTH_INITIALIZE,
-	       BE_AUTH_IGNORE
-	} state;
-};
-
-
-/* Reauthentication Timer state machine */
-struct eapol_reauth_timer_sm {
-	/* constants */
-	unsigned int reAuthPeriod; /* default 3600 s */
-	Boolean reAuthEnabled;
-
-	enum { REAUTH_TIMER_INITIALIZE, REAUTH_TIMER_REAUTHENTICATE } state;
-};
-
-
-/* Authenticator Key Transmit state machine */
-struct eapol_auth_key_tx {
-	enum { AUTH_KEY_TX_NO_KEY_TRANSMIT, AUTH_KEY_TX_KEY_TRANSMIT } state;
-};
-
-
-/* Key Receive state machine */
-struct eapol_key_rx {
-	/* variables */
-	Boolean rxKey;
-
-	enum { KEY_RX_NO_KEY_RECEIVE, KEY_RX_KEY_RECEIVE } state;
-};
-
-
-/* Controlled Directions state machine */
-struct eapol_ctrl_dir {
-	/* variables */
-	ControlledDirection adminControlledDirections;
-	ControlledDirection operControlledDirections;
-	Boolean operEdge;
-
-	enum { CTRL_DIR_FORCE_BOTH, CTRL_DIR_IN_OR_BOTH } state;
-};
-
-
 struct eap_sm;
 
 struct radius_attr_data {
@@ -115,6 +18,11 @@ struct radius_attr_data {
 	size_t len;
 };
 
+struct radius_class_data {
+	struct radius_attr_data *attr;
+	size_t count;
+};
+
 struct eapol_state_machine {
 	/* timers */
 	int aWhile;
@@ -145,12 +53,76 @@ struct eapol_state_machine {
 	/* Port Timers state machine */
 	/* 'Boolean tick' implicitly handled as registered timeout */
 
-	struct eapol_auth_pae_sm auth_pae;
-	struct eapol_backend_auth_sm be_auth;
-	struct eapol_reauth_timer_sm reauth_timer;
-	struct eapol_auth_key_tx auth_key_tx;
-	struct eapol_key_rx key_rx;
-	struct eapol_ctrl_dir ctrl_dir;
+	/* Authenticator PAE state machine */
+	enum { AUTH_PAE_INITIALIZE, AUTH_PAE_DISCONNECTED, AUTH_PAE_CONNECTING,
+	       AUTH_PAE_AUTHENTICATING, AUTH_PAE_AUTHENTICATED,
+	       AUTH_PAE_ABORTING, AUTH_PAE_HELD, AUTH_PAE_FORCE_AUTH,
+	       AUTH_PAE_FORCE_UNAUTH, AUTH_PAE_RESTART } auth_pae_state;
+	/* variables */
+	Boolean eapolLogoff;
+	Boolean eapolStart;
+	Boolean eapRestart;
+	PortTypes portMode;
+	unsigned int reAuthCount;
+	/* constants */
+	unsigned int quietPeriod; /* default 60; 0..65535 */
+#define AUTH_PAE_DEFAULT_quietPeriod 60
+	unsigned int reAuthMax; /* default 2 */
+#define AUTH_PAE_DEFAULT_reAuthMax 2
+	/* counters */
+	Counter authEntersConnecting;
+	Counter authEapLogoffsWhileConnecting;
+	Counter authEntersAuthenticating;
+	Counter authAuthSuccessesWhileAuthenticating;
+	Counter authAuthTimeoutsWhileAuthenticating;
+	Counter authAuthFailWhileAuthenticating;
+	Counter authAuthEapStartsWhileAuthenticating;
+	Counter authAuthEapLogoffWhileAuthenticating;
+	Counter authAuthReauthsWhileAuthenticated;
+	Counter authAuthEapStartsWhileAuthenticated;
+	Counter authAuthEapLogoffWhileAuthenticated;
+
+	/* Backend Authentication state machine */
+	enum { BE_AUTH_REQUEST, BE_AUTH_RESPONSE, BE_AUTH_SUCCESS,
+	       BE_AUTH_FAIL, BE_AUTH_TIMEOUT, BE_AUTH_IDLE, BE_AUTH_INITIALIZE,
+	       BE_AUTH_IGNORE
+	} be_auth_state;
+	/* variables */
+	Boolean eapNoReq;
+	Boolean eapReq;
+	Boolean eapResp;
+	/* constants */
+	unsigned int serverTimeout; /* default 30; 1..X */
+#define BE_AUTH_DEFAULT_serverTimeout 30
+	/* counters */
+	Counter backendResponses;
+	Counter backendAccessChallenges;
+	Counter backendOtherRequestsToSupplicant;
+	Counter backendAuthSuccesses;
+	Counter backendAuthFails;
+
+	/* Reauthentication Timer state machine */
+	enum { REAUTH_TIMER_INITIALIZE, REAUTH_TIMER_REAUTHENTICATE
+	} reauth_timer_state;
+	/* constants */
+	unsigned int reAuthPeriod; /* default 3600 s */
+	Boolean reAuthEnabled;
+
+	/* Authenticator Key Transmit state machine */
+	enum { AUTH_KEY_TX_NO_KEY_TRANSMIT, AUTH_KEY_TX_KEY_TRANSMIT
+	} auth_key_tx_state;
+
+	/* Key Receive state machine */
+	enum { KEY_RX_NO_KEY_RECEIVE, KEY_RX_KEY_RECEIVE } key_rx_state;
+	/* variables */
+	Boolean rxKey;
+
+	/* Controlled Directions state machine */
+	enum { CTRL_DIR_FORCE_BOTH, CTRL_DIR_IN_OR_BOTH } ctrl_dir_state;
+	/* variables */
+	ControlledDirection adminControlledDirections;
+	ControlledDirection operControlledDirections;
+	Boolean operEdge;
 
 	/* Authenticator Statistics Table */
 	Counter dot1xAuthEapolFramesRx;
@@ -180,8 +152,7 @@ struct eapol_state_machine {
 	size_t last_eap_radius_len;
 	u8 *identity;
 	size_t identity_len;
-	struct radius_attr_data *radius_class;
-	size_t radius_class_count;
+	struct radius_class_data radius_class;
 
 	/* Keys for encrypting and signing EAPOL-Key frames */
 	u8 *eapol_key_sign;
@@ -201,6 +172,7 @@ struct eapol_state_machine {
 	u8 currentId;
 
 	Boolean initializing; /* in process of initializing state machines */
+	Boolean changed;
 
 	/* Somewhat nasty pointers to global hostapd and STA data to avoid
 	 * passing these to every function */
@@ -216,5 +188,6 @@ void eapol_sm_step(struct eapol_state_machine *sm);
 void eapol_sm_initialize(struct eapol_state_machine *sm);
 void eapol_sm_dump_state(FILE *f, const char *prefix,
 			 struct eapol_state_machine *sm);
+int eapol_sm_eap_pending_cb(struct eapol_state_machine *sm, void *ctx);
 
 #endif /* EAPOL_SM_H */
