@@ -13,18 +13,8 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <string.h>
-#include <signal.h>
-#include <time.h>
+#include "includes.h"
 #include <syslog.h>
-#include <stdarg.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
 
 #include "eloop.h"
 #include "hostapd.h"
@@ -40,6 +30,7 @@
 #include "radius_client.h"
 #include "radius_server.h"
 #include "wpa.h"
+#include "preauth.h"
 #include "ctrl_iface.h"
 #include "tls.h"
 #include "eap_sim_db.h"
@@ -221,12 +212,31 @@ void hostapd_new_assoc_sta(hostapd *hapd, struct sta_info *sta, int reassoc)
 	/* Start IEEE 802.1X authentication process for new stations */
 	ieee802_1x_new_station(hapd, sta);
 	if (reassoc)
-		wpa_sm_event(hapd, sta, WPA_REAUTH);
+		wpa_auth_sm_event(sta->wpa_sm, WPA_REAUTH);
 	else
-		wpa_new_station(hapd, sta);
+		wpa_auth_sta_associated(hapd->wpa_auth, sta->wpa_sm);
 }
 
 
+#ifdef EAP_SERVER
+static int hostapd_sim_db_cb_sta(struct hostapd_data *hapd,
+				 struct sta_info *sta, void *ctx)
+{
+	if (eapol_sm_eap_pending_cb(sta->eapol_sm, ctx) == 0)
+		return 1;
+	return 0;
+}
+
+
+static void hostapd_sim_db_cb(void *ctx, void *session_ctx)
+{
+	struct hostapd_data *hapd = ctx;
+	if (ap_for_each_sta(hapd, hostapd_sim_db_cb_sta, session_ctx) == 0)
+		radius_server_eap_pending_cb(hapd->radius_srv, session_ctx);
+}
+#endif /* EAP_SERVER */
+
+
 static void handle_term(int sig, void *eloop_ctx, void *signal_ctx)
 {
 	printf("Signal %d received - terminating\n", sig);
@@ -234,11 +244,27 @@ static void handle_term(int sig, void *eloop_ctx, void *signal_ctx)
 }
 
 
+static void hostapd_wpa_auth_conf(struct hostapd_config *conf,
+				  struct wpa_auth_config *wconf)
+{
+	wconf->wpa = conf->wpa;
+	wconf->wpa_key_mgmt = conf->wpa_key_mgmt;
+	wconf->wpa_pairwise = conf->wpa_pairwise;
+	wconf->wpa_group = conf->wpa_group;
+	wconf->wpa_group_rekey = conf->wpa_group_rekey;
+	wconf->wpa_strict_rekey = conf->wpa_strict_rekey;
+	wconf->wpa_gmk_rekey = conf->wpa_gmk_rekey;
+	wconf->rsn_preauth = conf->rsn_preauth;
+	wconf->stakey = conf->stakey;
+}
+
+
 static void handle_reload(int sig, void *eloop_ctx, void *signal_ctx)
 {
 	struct hapd_interfaces *hapds = (struct hapd_interfaces *) eloop_ctx;
 	struct hostapd_config *newconf;
 	int i;
+	struct wpa_auth_config wpa_auth_conf;
 
 	printf("Signal %d received - reloading configuration\n", sig);
 
@@ -255,6 +281,10 @@ static void handle_reload(int sig, void *eloop_ctx, void *signal_ctx)
 		 * deny list, etc.) */
 		radius_client_flush(hapd->radius);
 		hostapd_config_free(hapd->conf);
+
+		hostapd_wpa_auth_conf(newconf, &wpa_auth_conf);
+		wpa_reconfig(hapd->wpa_auth, &wpa_auth_conf);
+
 		hapd->conf = newconf;
 	}
 }
@@ -368,7 +398,23 @@ static void hostapd_cleanup(struct hostapd_data *hapd)
 	hapd->default_wep_key = NULL;
 	iapp_deinit(hapd->iapp);
 	accounting_deinit(hapd);
-	wpa_deinit(hapd);
+	rsn_preauth_iface_deinit(hapd);
+	if (hapd->wpa_auth) {
+		wpa_deinit(hapd->wpa_auth);
+		hapd->wpa_auth = NULL;
+
+		if (hostapd_set_privacy(hapd, 0)) {
+			wpa_printf(MSG_DEBUG, "Could not disable "
+				   "PrivacyInvoked for interface %s",
+				   hapd->conf->iface);
+		}
+
+		if (hostapd_set_generic_elem(hapd, (u8 *) "", 0)) {
+			wpa_printf(MSG_DEBUG, "Could not remove generic "
+				   "information element from interface %s",
+				   hapd->conf->iface);
+		}
+	}
 	ieee802_1x_deinit(hapd);
 	hostapd_acl_deinit(hapd);
 	radius_client_deinit(hapd->radius);
@@ -393,8 +439,10 @@ static void hostapd_cleanup(struct hostapd_data *hapd)
 	}
 #endif /* EAP_TLS_FUNCS */
 
+#ifdef EAP_SERVER
 	if (hapd->eap_sim_db_priv)
 		eap_sim_db_deinit(hapd->eap_sim_db_priv);
+#endif /* EAP_SERVER */
 }
 
 
@@ -414,6 +462,188 @@ static int hostapd_flush_old_stations(hostapd *hapd)
 }
 
 
+static void hostapd_wpa_auth_logger(void *ctx, const u8 *addr,
+				    logger_level level, const char *txt)
+{
+	struct hostapd_data *hapd = ctx;
+	int hlevel;
+
+	switch (level) {
+	case LOGGER_WARNING:
+		hlevel = HOSTAPD_LEVEL_WARNING;
+		break;
+	case LOGGER_INFO:
+		hlevel = HOSTAPD_LEVEL_INFO;
+		break;
+	case LOGGER_DEBUG:
+	default:
+		hlevel = HOSTAPD_LEVEL_DEBUG;
+		break;
+	}
+
+	hostapd_logger(hapd, addr, HOSTAPD_MODULE_WPA, hlevel, "%s", txt);
+}
+
+
+static void hostapd_wpa_auth_disconnect(void *ctx, const u8 *addr,
+					u16 reason)
+{
+	struct hostapd_data *hapd = ctx;
+	struct sta_info *sta;
+
+	wpa_printf(MSG_DEBUG, "%s: WPA authenticator requests disconnect: "
+		   "STA " MACSTR " reason %d",
+		   __func__, MAC2STR(addr), reason);
+
+	sta = ap_get_sta(hapd, addr);
+	hostapd_sta_deauth(hapd, addr, reason);
+	if (sta == NULL)
+		return;
+	sta->flags &= ~(WLAN_STA_AUTH | WLAN_STA_ASSOC | WLAN_STA_AUTHORIZED);
+	eloop_cancel_timeout(ap_handle_timer, hapd, sta);
+	eloop_register_timeout(0, 0, ap_handle_timer, hapd, sta);
+	sta->timeout_next = STA_REMOVE;
+}
+
+
+static void hostapd_wpa_auth_mic_failure_report(void *ctx, const u8 *addr)
+{
+	struct hostapd_data *hapd = ctx;
+	ieee80211_michael_mic_failure(hapd, addr, 0);
+}
+
+
+static void hostapd_wpa_auth_set_eapol(void *ctx, const u8 *addr,
+				       wpa_eapol_variable var, int value)
+{
+	struct hostapd_data *hapd = ctx;
+	struct sta_info *sta = ap_get_sta(hapd, addr);
+	if (sta == NULL)
+		return;
+	switch (var) {
+	case WPA_EAPOL_portEnabled:
+		ieee802_1x_notify_port_enabled(sta->eapol_sm, value);
+		break;
+	case WPA_EAPOL_portValid:
+		ieee802_1x_notify_port_valid(sta->eapol_sm, value);
+		break;
+	case WPA_EAPOL_authorized:
+		ieee802_1x_set_sta_authorized(hapd, sta, value);
+		break;
+	case WPA_EAPOL_portControl_Auto:
+		if (sta->eapol_sm)
+			sta->eapol_sm->portControl = Auto;
+		break;
+	case WPA_EAPOL_keyRun:
+		if (sta->eapol_sm)
+			sta->eapol_sm->keyRun = value ? TRUE : FALSE;
+		break;
+	case WPA_EAPOL_keyAvailable:
+		if (sta->eapol_sm)
+			sta->eapol_sm->keyAvailable = value ? TRUE : FALSE;
+		break;
+	case WPA_EAPOL_keyDone:
+		if (sta->eapol_sm)
+			sta->eapol_sm->keyDone = value ? TRUE : FALSE;
+		break;
+	case WPA_EAPOL_inc_EapolFramesTx:
+		if (sta->eapol_sm)
+			sta->eapol_sm->dot1xAuthEapolFramesTx++;
+		break;
+	}
+}
+
+
+static int hostapd_wpa_auth_get_eapol(void *ctx, const u8 *addr,
+				      wpa_eapol_variable var)
+{
+	struct hostapd_data *hapd = ctx;
+	struct sta_info *sta = ap_get_sta(hapd, addr);
+	if (sta == NULL || sta->eapol_sm == NULL)
+		return -1;
+	switch (var) {
+	case WPA_EAPOL_keyRun:
+		return sta->eapol_sm->keyRun;
+	case WPA_EAPOL_keyAvailable:
+		return sta->eapol_sm->keyAvailable;
+	default:
+		return -1;
+	}
+}
+
+
+static const u8 * hostapd_wpa_auth_get_psk(void *ctx, const u8 *addr,
+					   const u8 *prev_psk)
+{
+	struct hostapd_data *hapd = ctx;
+	return hostapd_get_psk(hapd->conf, addr, prev_psk);
+}
+
+
+static int hostapd_wpa_auth_get_pmk(void *ctx, const u8 *addr, u8 *pmk,
+				    size_t *len)
+{
+	struct hostapd_data *hapd = ctx;
+	u8 *key;
+	size_t keylen;
+	struct sta_info *sta;
+
+	sta = ap_get_sta(hapd, addr);
+	if (sta == NULL)
+		return -1;
+
+	key = ieee802_1x_get_key_crypt(sta->eapol_sm, &keylen);
+	if (key == NULL)
+		return -1;
+
+	if (keylen > *len)
+		keylen = WPA_PMK_LEN;
+	memcpy(pmk, key, keylen);
+	*len = keylen;
+	return 0;
+}
+
+
+static int hostapd_wpa_auth_set_key(void *ctx, const char *alg, const u8 *addr,
+				    int idx, u8 *key, size_t key_len)
+{
+	struct hostapd_data *hapd = ctx;
+	return hostapd_set_encryption(hapd, alg, addr, idx, key, key_len);
+}
+
+
+static int hostapd_wpa_auth_get_seqnum(void *ctx, const u8 *addr, int idx,
+				       u8 *seq)
+{
+	struct hostapd_data *hapd = ctx;
+	return hostapd_get_seqnum(hapd, addr, idx, seq);
+}
+
+
+static int hostapd_wpa_auth_send_eapol(void *ctx, const u8 *addr,
+				       const u8 *data, size_t data_len,
+				       int encrypt)
+{
+	struct hostapd_data *hapd = ctx;
+	return hostapd_send_eapol(hapd, addr, data, data_len, encrypt);
+}
+
+
+static int hostapd_wpa_auth_for_each_sta(
+	void *ctx, int (*cb)(struct wpa_state_machine *sm, void *ctx),
+	void *cb_ctx)
+{
+	struct hostapd_data *hapd = ctx;
+	struct sta_info *sta;
+
+	for (sta = hapd->sta_list; sta; sta = sta->next) {
+		if (sta->wpa_sm && cb(sta->wpa_sm, cb_ctx))
+			return 1;
+	}
+	return 0;
+}
+
+
 static int hostapd_setup_interface(struct hostapd_data *hapd)
 {
 	struct hostapd_config *conf = hapd->conf;
@@ -504,9 +734,50 @@ static int hostapd_setup_interface(struct hostapd_data *hapd)
 		return -1;
 	}
 
-	if (hapd->conf->wpa && wpa_init(hapd)) {
-		printf("WPA initialization failed.\n");
-		return -1;
+	if (hapd->conf->wpa) {
+		struct wpa_auth_config conf;
+		struct wpa_auth_callbacks cb;
+		const u8 *wpa_ie;
+		size_t wpa_ie_len;
+
+		hostapd_wpa_auth_conf(hapd->conf, &conf);
+		memset(&cb, 0, sizeof(cb));
+		cb.ctx = hapd;
+		cb.logger = hostapd_wpa_auth_logger;
+		cb.disconnect = hostapd_wpa_auth_disconnect;
+		cb.mic_failure_report = hostapd_wpa_auth_mic_failure_report;
+		cb.set_eapol = hostapd_wpa_auth_set_eapol;
+		cb.get_eapol = hostapd_wpa_auth_get_eapol;
+		cb.get_psk = hostapd_wpa_auth_get_psk;
+		cb.get_pmk = hostapd_wpa_auth_get_pmk;
+		cb.set_key = hostapd_wpa_auth_set_key;
+		cb.get_seqnum = hostapd_wpa_auth_get_seqnum;
+		cb.send_eapol = hostapd_wpa_auth_send_eapol;
+		cb.for_each_sta = hostapd_wpa_auth_for_each_sta;
+		hapd->wpa_auth = wpa_init(hapd->own_addr, &conf, &cb);
+		if (hapd->wpa_auth == NULL) {
+			printf("WPA initialization failed.\n");
+			return -1;
+		}
+
+		if (hostapd_set_privacy(hapd, 1)) {
+			wpa_printf(MSG_ERROR, "Could not set PrivacyInvoked "
+				   "for interface %s", hapd->conf->iface);
+			return -1;
+		}
+
+		wpa_ie = wpa_auth_get_wpa_ie(hapd->wpa_auth, &wpa_ie_len);
+		if (hostapd_set_generic_elem(hapd, wpa_ie, wpa_ie_len)) {
+			wpa_printf(MSG_ERROR, "Failed to configure WPA IE for "
+				   "the kernel driver.");
+			return -1;
+		}
+
+		if (rsn_preauth_iface_init(hapd)) {
+			printf("Initialization of RSN pre-authentication "
+			       "failed.\n");
+			return -1;
+		}
 	}
 
 	if (accounting_init(hapd)) {
@@ -705,15 +976,18 @@ static hostapd * hostapd_init(const char *config_file)
 	}
 #endif /* EAP_TLS_FUNCS */
 
+#ifdef EAP_SERVER
 	if (hapd->conf->eap_sim_db) {
 		hapd->eap_sim_db_priv =
-			eap_sim_db_init(hapd->conf->eap_sim_db);
+			eap_sim_db_init(hapd->conf->eap_sim_db,
+					hostapd_sim_db_cb, hapd);
 		if (hapd->eap_sim_db_priv == NULL) {
 			printf("Failed to initialize EAP-SIM database "
 			       "interface\n");
 			goto fail;
 		}
 	}
+#endif /* EAP_SERVER */
 
 	if (hapd->conf->assoc_ap)
 		hapd->assoc_ap_state = WAIT_BEACON;
@@ -788,7 +1062,11 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	eloop_init(&interfaces);
+	if (eloop_init(&interfaces)) {
+		wpa_printf(MSG_ERROR, "Failed to initialize event loop");
+		return -1;
+	}
+
 	eloop_register_signal(SIGHUP, handle_reload, NULL);
 	eloop_register_signal(SIGINT, handle_term, NULL);
 	eloop_register_signal(SIGTERM, handle_term, NULL);
