@@ -13,15 +13,30 @@
 #ifndef ETH_P_ALL
 #define ETH_P_ALL 0x0003
 #endif
+#ifndef ETH_P_PAE
+#define ETH_P_PAE 0x888E /* Port Access Entity (IEEE 802.1X) */
+#endif /* ETH_P_PAE */
+
+#ifndef BIT
+#define BIT(x) (1 << (x))
+#endif
+
+#ifndef MAC2STR
+#define MAC2STR(a) (a)[0], (a)[1], (a)[2], (a)[3], (a)[4], (a)[5]
+#define MACSTR "%02x:%02x:%02x:%02x:%02x:%02x"
+#endif
 
-#include "hostap_common.h"
 #include "config.h"
 
+#ifdef _MSC_VER
+#pragma pack(push, 1)
+#endif /* _MSC_VER */
+
 struct ieee8023_hdr {
 	u8 dest[6];
 	u8 src[6];
 	u16 ethertype;
-} __attribute__ ((packed));
+} STRUCT_PACKED;
 
 
 struct ieee80211_hdr {
@@ -33,7 +48,11 @@ struct ieee80211_hdr {
 	u16 seq_ctrl;
 	/* followed by 'u8 addr4[6];' if ToDS and FromDS is set in data frame
 	 */
-} __attribute__ ((packed));
+} STRUCT_PACKED;
+
+#ifdef _MSC_VER
+#pragma pack(pop)
+#endif /* _MSC_VER */
 
 #define IEEE80211_DA_FROMDS addr1
 #define IEEE80211_BSSID_FROMDS addr2
@@ -95,16 +114,8 @@ struct hostapd_data {
 	struct hostapd_cached_radius_acl *acl_cache;
 	struct hostapd_acl_query_data *acl_queries;
 
-	u8 *wpa_ie;
-	size_t wpa_ie_len;
 	struct wpa_authenticator *wpa_auth;
 
-#define PMKID_HASH_SIZE 128
-#define PMKID_HASH(pmkid) (unsigned int) ((pmkid)[0] & 0x7f)
-	struct rsn_pmksa_cache *pmkid[PMKID_HASH_SIZE];
-	struct rsn_pmksa_cache *pmksa;
-	int pmksa_count;
-
 	struct rsn_preauth_interface *preauth_iface;
 	time_t michael_mic_failure;
 	int michael_mic_failures;
@@ -118,17 +129,19 @@ struct hostapd_data {
 	struct radius_server_data *radius_srv;
 };
 
-void hostapd_new_assoc_sta(hostapd *hapd, struct sta_info *sta);
+void hostapd_new_assoc_sta(hostapd *hapd, struct sta_info *sta, int reassoc);
 void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
 		    unsigned int module, int level, const char *fmt,
-		    ...) __attribute__ ((format (printf, 5, 6)));
+		    ...) PRINTF_FORMAT(5, 6);
 
 
+#ifndef _MSC_VER
 #define HOSTAPD_DEBUG(level, args...) \
 do { \
 	if (hapd->conf == NULL || hapd->conf->debug >= (level)) \
 		printf(args); \
 } while (0)
+#endif /* _MSC_VER */
 
 #define HOSTAPD_DEBUG_COND(level) (hapd->conf->debug >= (level))
 
