@@ -13,14 +13,8 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <sys/socket.h>
+#include "includes.h"
 #include <net/if.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <time.h>
 
 #include "eloop.h"
 #include "hostapd.h"
@@ -92,8 +86,7 @@ ParseRes ieee802_11_parse_elems(struct hostapd_data *hapd, u8 *start,
 					      "failed (id=%d elen=%d "
 					      "left=%lu)\n",
 					      id, elen, (unsigned long) left);
-				if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MINIMAL))
-					hostapd_hexdump("IEs", start, len);
+				wpa_hexdump(MSG_MSGDUMP, "IEs", start, len);
 			}
 			return ParseFailed;
 		}
@@ -320,7 +313,7 @@ static u16 auth_shared_key(hostapd *hapd, struct sta_info *sta,
 	 * authentication reply. */
 #else
 	sta->flags |= WLAN_STA_AUTH;
-	wpa_sm_event(hapd, sta, WPA_AUTH);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_AUTH);
 #endif
 	free(sta->challenge);
 	sta->challenge = NULL;
@@ -497,7 +490,7 @@ static void handle_auth(hostapd *hapd, struct ieee80211_mgmt *mgmt, size_t len)
 		 * authentication reply. */
 #else
 		sta->flags |= WLAN_STA_AUTH;
-		wpa_sm_event(hapd, sta, WPA_AUTH);
+		wpa_auth_sm_event(sta->wpa_sm, WPA_AUTH);
 #endif
 		break;
 	case WLAN_AUTH_SHARED_KEY:
@@ -642,28 +635,28 @@ static void handle_assoc(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 		int res;
 		wpa_ie -= 2;
 		wpa_ie_len += 2;
-		res = wpa_validate_wpa_ie(hapd, sta, wpa_ie, wpa_ie_len,
-					  elems.rsn_ie ?
-					  HOSTAPD_WPA_VERSION_WPA2 :
-					  HOSTAPD_WPA_VERSION_WPA);
+		if (sta->wpa_sm == NULL)
+			sta->wpa_sm = wpa_auth_sta_init(hapd->wpa_auth,
+							sta->addr);
+		if (sta->wpa_sm == NULL) {
+			printf("Failed to initialize WPA state machine\n");
+			resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
+			goto fail;
+		}
+		res = wpa_validate_wpa_ie(hapd->wpa_auth, sta->wpa_sm,
+					  wpa_ie, wpa_ie_len);
 		if (res == WPA_INVALID_GROUP)
 			resp = WLAN_STATUS_GROUP_CIPHER_NOT_VALID;
 		else if (res == WPA_INVALID_PAIRWISE)
 			resp = WLAN_STATUS_PAIRWISE_CIPHER_NOT_VALID;
 		else if (res == WPA_INVALID_AKMP)
 			resp = WLAN_STATUS_AKMP_NOT_VALID;
+		else if (res == WPA_ALLOC_FAIL)
+			resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
 		else if (res != WPA_IE_OK)
 			resp = WLAN_STATUS_INVALID_IE;
 		if (resp != WLAN_STATUS_SUCCESS)
 			goto fail;
-		free(sta->wpa_ie);
-		sta->wpa_ie = malloc(wpa_ie_len);
-		if (sta->wpa_ie == NULL) {
-			resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
-			goto fail;
-		}
-		sta->wpa_ie_len = wpa_ie_len;
-		memcpy(sta->wpa_ie, wpa_ie, wpa_ie_len);
 	}
 
 	/* get a unique AID */
@@ -828,11 +821,11 @@ static void handle_disassoc(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 	}
 
 	sta->flags &= ~WLAN_STA_ASSOC;
-	wpa_sm_event(hapd, sta, WPA_DISASSOC);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_DISASSOC);
 	hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE80211,
 		       HOSTAPD_LEVEL_INFO, "disassociated");
 	sta->acct_terminate_cause = RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-	ieee802_1x_set_port_enabled(hapd, sta, 0);
+	ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 	/* Stop Accounting and IEEE 802.1X sessions, but leave the STA
 	 * authenticated. */
 	accounting_sta_stop(hapd, sta);
@@ -885,11 +878,11 @@ static void handle_deauth(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 	}
 
 	sta->flags &= ~(WLAN_STA_AUTH | WLAN_STA_ASSOC);
-	wpa_sm_event(hapd, sta, WPA_DEAUTH);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_DEAUTH);
 	hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE80211,
 		       HOSTAPD_LEVEL_DEBUG, "deauthenticated");
 	sta->acct_terminate_cause = RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-	ieee802_1x_set_port_enabled(hapd, sta, 0);
+	ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 	ap_free_sta(hapd, sta);
 }
 
@@ -1094,7 +1087,7 @@ static void handle_assoc_cb(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 		printf("Could not add station to kernel driver.\n");
 	}
 
-	wpa_sm_event(hapd, sta, WPA_ASSOC);
+	wpa_auth_sm_event(sta->wpa_sm, WPA_ASSOC);
 	hostapd_new_assoc_sta(hapd, sta, !new_assoc);
 
 	ieee802_1x_notify_port_enabled(sta->eapol_sm, 1);
@@ -1140,6 +1133,7 @@ static void ieee80211_tkip_countermeasures_stop(void *eloop_ctx,
 {
 	struct hostapd_data *hapd = eloop_ctx;
 	hapd->tkip_countermeasures = 0;
+	hostapd_set_countermeasures(hapd, 0);
 	hostapd_logger(hapd, NULL, HOSTAPD_MODULE_IEEE80211,
 		       HOSTAPD_LEVEL_INFO, "TKIP countermeasures ended");
 }
@@ -1152,10 +1146,10 @@ static void ieee80211_tkip_countermeasures_start(struct hostapd_data *hapd)
 	hostapd_logger(hapd, NULL, HOSTAPD_MODULE_IEEE80211,
 		       HOSTAPD_LEVEL_INFO, "TKIP countermeasures initiated");
 
-	if (hapd->wpa_auth)
-		hapd->wpa_auth->dot11RSNATKIPCounterMeasuresInvoked++;
+	wpa_auth_countermeasures_start(hapd->wpa_auth);
 	hapd->tkip_countermeasures = 1;
-	wpa_gtk_rekey(hapd);
+	hostapd_set_countermeasures(hapd, 1);
+	wpa_gtk_rekey(hapd->wpa_auth);
 	eloop_cancel_timeout(ieee80211_tkip_countermeasures_stop, hapd, NULL);
 	eloop_register_timeout(60, 0, ieee80211_tkip_countermeasures_stop,
 			       hapd, NULL);
@@ -1169,7 +1163,7 @@ static void ieee80211_tkip_countermeasures_start(struct hostapd_data *hapd)
 }
 
 
-void ieee80211_michael_mic_failure(struct hostapd_data *hapd, u8 *addr,
+void ieee80211_michael_mic_failure(struct hostapd_data *hapd, const u8 *addr,
 				   int local)
 {
 	time_t now;
@@ -1177,7 +1171,7 @@ void ieee80211_michael_mic_failure(struct hostapd_data *hapd, u8 *addr,
 	if (addr && local) {
 		struct sta_info *sta = ap_get_sta(hapd, addr);
 		if (sta != NULL) {
-			sta->dot11RSNAStatsTKIPLocalMICFailures++;
+			wpa_auth_sta_local_mic_failure_report(sta->wpa_sm);
 			hostapd_logger(hapd, addr, HOSTAPD_MODULE_IEEE80211,
 				       HOSTAPD_LEVEL_INFO,
 				       "Michael MIC failure detected in "
