@@ -13,17 +13,9 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <string.h>
+#include "includes.h"
 #include <sys/ioctl.h>
-#include <signal.h>
 #include <assert.h>
-#include <time.h>
-#include <sys/time.h>
-#include <sys/socket.h>
 
 #include "hostapd.h"
 #include "ieee802_1x.h"
@@ -36,6 +28,8 @@
 #include "eloop.h"
 #include "sta_info.h"
 #include "wpa.h"
+#include "preauth.h"
+#include "pmksa_cache.h"
 #include "driver.h"
 #include "eap.h"
 
@@ -81,7 +75,7 @@ static void ieee802_1x_send(hostapd *hapd, struct sta_info *sta, u8 type,
 	if (datalen > 0 && data != NULL)
 		memcpy(xhdr + 1, data, datalen);
 
-	if (sta->wpa_sm && sta->wpa_sm->pairwise_set)
+	if (wpa_auth_pairwise_set(sta->wpa_sm))
 		encrypt = 1;
 	if (sta->flags & WLAN_STA_PREAUTH) {
 		rsn_preauth_send(hapd, sta, buf, len);
@@ -144,7 +138,7 @@ void ieee802_1x_request_identity(struct hostapd_data *hapd,
 		return;
 	}
 
-	if (sm == NULL || !sm->auth_pae.eapRestart)
+	if (sm == NULL || !sm->eapRestart)
 		return;
 
 	ieee802_1x_new_auth_session(hapd, sta);
@@ -170,7 +164,7 @@ void ieee802_1x_request_identity(struct hostapd_data *hapd,
 		       hapd->conf->eap_req_id_text_len);
 	}
 
-	sm->be_auth.eapReq = TRUE;
+	sm->eapReq = TRUE;
 	free(sm->last_eap_radius);
 	sm->last_eap_radius = buf;
 	sm->last_eap_radius_len = tlen;
@@ -184,7 +178,7 @@ void ieee802_1x_request_identity(struct hostapd_data *hapd,
 		      " (identifier %d, timeout 30)\n", MAC2STR(sta->addr),
 		      eap->identifier);
 
-	sm->auth_pae.eapRestart = FALSE;
+	sm->eapRestart = FALSE;
 }
 
 
@@ -248,22 +242,6 @@ void ieee802_1x_tx_req(struct hostapd_data *hapd, struct sta_info *sta)
 }
 
 
-void hostapd_get_ntp_timestamp(u8 *buf)
-{
-	struct timeval now;
-	u32 sec, usec;
-
-	/* 64-bit NTP timestamp (time from 1900-01-01 00:00:00) */
-	gettimeofday(&now, NULL);
-	sec = htonl(now.tv_sec + 2208988800U); /* Epoch to 1900 */
-	/* Estimate 2^32/10^6 = 4295 - 1/32 - 1/512 */
-	usec = now.tv_usec;
-	usec = htonl(4295 * usec - (usec >> 5) - (usec >> 9));
-	memcpy(buf, (u8 *) &sec, 4);
-	memcpy(buf + 4, (u8 *) &usec, 4);
-}
-
-
 static void ieee802_1x_tx_key_one(hostapd *hapd, struct sta_info *sta,
 				  int index, int broadcast,
 				  u8 *key_data, size_t key_len)
@@ -287,7 +265,7 @@ static void ieee802_1x_tx_key_one(hostapd *hapd, struct sta_info *sta,
 	key = (struct ieee802_1x_eapol_key *) (hdr + 1);
 	key->type = EAPOL_KEY_TYPE_RC4;
 	key->key_length = htons(key_len);
-	hostapd_get_ntp_timestamp(key->replay_counter);
+	wpa_get_ntp_timestamp(key->replay_counter);
 
 	if (hostapd_get_rand(key->key_iv, sizeof(key->key_iv))) {
 		printf("Could not get random numbers\n");
@@ -368,10 +346,8 @@ void ieee802_1x_tx_key(struct hostapd_data *hapd, struct sta_info *sta)
 			return;
 		}
 
-		if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MINIMAL))
-			hostapd_hexdump("Individual WEP key",
-					ikey,
-					hapd->conf->individual_wep_key_len);
+		wpa_hexdump_key(MSG_DEBUG, "Individual WEP key",
+				ikey, hapd->conf->individual_wep_key_len);
 
 		ieee802_1x_tx_key_one(hapd, sta, 0, 0, ikey,
 				      hapd->conf->individual_wep_key_len);
@@ -677,6 +653,7 @@ void ieee802_1x_receive(struct hostapd_data *hapd, const u8 *sa, const u8 *buf,
 	struct ieee802_1x_hdr *hdr;
 	struct ieee802_1x_eapol_key *key;
 	u16 datalen;
+	struct rsn_pmksa_cache_entry *pmksa;
 
 	if (!hapd->conf->ieee802_1x && !hapd->conf->wpa)
 		return;
@@ -724,11 +701,13 @@ void ieee802_1x_receive(struct hostapd_data *hapd, const u8 *sa, const u8 *buf,
 	    hdr->type == IEEE802_1X_TYPE_EAPOL_KEY &&
 	    (key->type == EAPOL_KEY_TYPE_WPA ||
 	     key->type == EAPOL_KEY_TYPE_RSN)) {
-		wpa_receive(hapd, sta, (u8 *) hdr, sizeof(*hdr) + datalen);
+		wpa_receive(hapd->wpa_auth, sta->wpa_sm, (u8 *) hdr,
+			    sizeof(*hdr) + datalen);
 		return;
 	}
 
-	if (!hapd->conf->ieee802_1x || sta->wpa_key_mgmt == WPA_KEY_MGMT_PSK)
+	if (!hapd->conf->ieee802_1x ||
+	    wpa_auth_sta_key_mgmt(sta->wpa_sm) == WPA_KEY_MGMT_PSK)
 		return;
 
 	if (!sta->eapol_sm) {
@@ -753,16 +732,17 @@ void ieee802_1x_receive(struct hostapd_data *hapd, const u8 *sa, const u8 *buf,
 		hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE8021X,
 			       HOSTAPD_LEVEL_DEBUG, "received EAPOL-Start "
 			       "from STA");
-		if (sta->pmksa) {
+		pmksa = wpa_auth_sta_get_pmksa(sta->wpa_sm);
+		if (pmksa) {
 			hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_WPA,
 				       HOSTAPD_LEVEL_DEBUG, "cached PMKSA "
 				       "available - ignore it since "
 				       "STA sent EAPOL-Start");
-			sta->pmksa = NULL;
+			wpa_auth_sta_clear_pmksa(sta->wpa_sm, pmksa);
 		}
-		sta->eapol_sm->auth_pae.eapolStart = TRUE;
+		sta->eapol_sm->eapolStart = TRUE;
 		sta->eapol_sm->dot1xAuthEapolStartFramesRx++;
-		wpa_sm_event(hapd, sta, WPA_REAUTH_EAPOL);
+		wpa_auth_sm_event(sta->wpa_sm, WPA_REAUTH_EAPOL);
 		break;
 
 	case IEEE802_1X_TYPE_EAPOL_LOGOFF:
@@ -771,7 +751,7 @@ void ieee802_1x_receive(struct hostapd_data *hapd, const u8 *sa, const u8 *buf,
 			       "from STA");
 		sta->acct_terminate_cause =
 			RADIUS_ACCT_TERMINATE_CAUSE_USER_REQUEST;
-		sta->eapol_sm->auth_pae.eapolLogoff = TRUE;
+		sta->eapol_sm->eapolLogoff = TRUE;
 		sta->eapol_sm->dot1xAuthEapolLogoffFramesRx++;
 		break;
 
@@ -803,7 +783,10 @@ void ieee802_1x_receive(struct hostapd_data *hapd, const u8 *sa, const u8 *buf,
 
 void ieee802_1x_new_station(hostapd *hapd, struct sta_info *sta)
 {
-	if (!hapd->conf->ieee802_1x || sta->wpa_key_mgmt == WPA_KEY_MGMT_PSK)
+	struct rsn_pmksa_cache_entry *pmksa;
+
+	if (!hapd->conf->ieee802_1x ||
+	    wpa_auth_sta_key_mgmt(sta->wpa_sm) == WPA_KEY_MGMT_PSK)
 		return;
 
 	if (sta->eapol_sm == NULL) {
@@ -821,7 +804,8 @@ void ieee802_1x_new_station(hostapd *hapd, struct sta_info *sta)
 
 	sta->eapol_sm->portEnabled = TRUE;
 
-	if (sta->pmksa) {
+	pmksa = wpa_auth_sta_get_pmksa(sta->wpa_sm);
+	if (pmksa) {
 		hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE8021X,
 			       HOSTAPD_LEVEL_DEBUG,
 			       "PMK from PMKSA cache - skip IEEE 802.1X/EAP");
@@ -829,11 +813,12 @@ void ieee802_1x_new_station(hostapd *hapd, struct sta_info *sta)
 		 * because of existing PMKSA information in the cache. */
 		sta->eapol_sm->keyRun = TRUE;
 		sta->eapol_sm->keyAvailable = TRUE;
-		sta->eapol_sm->auth_pae.state = AUTH_PAE_AUTHENTICATING;
-		sta->eapol_sm->be_auth.state = BE_AUTH_SUCCESS;
+		sta->eapol_sm->auth_pae_state = AUTH_PAE_AUTHENTICATING;
+		sta->eapol_sm->be_auth_state = BE_AUTH_SUCCESS;
 		sta->eapol_sm->authSuccess = TRUE;
 		if (sta->eapol_sm->eap)
 			eap_sm_notify_cached(sta->eapol_sm->eap);
+		pmksa_cache_to_eapol_data(pmksa, sta->eapol_sm);
 	} else
 		eapol_sm_step(sta->eapol_sm);
 }
@@ -918,7 +903,7 @@ static void ieee802_1x_decapsulate_radius(hostapd *hapd, struct sta_info *sta)
 
 	if (sm == NULL || sm->last_recv_radius == NULL) {
 		if (sm)
-			sm->be_auth.eapNoReq = TRUE;
+			sm->eapNoReq = TRUE;
 		return;
 	}
 
@@ -935,7 +920,7 @@ static void ieee802_1x_decapsulate_radius(hostapd *hapd, struct sta_info *sta)
 		free(sm->last_eap_radius);
 		sm->last_eap_radius = NULL;
 		sm->last_eap_radius_len = 0;
-		sm->be_auth.eapNoReq = TRUE;
+		sm->eapNoReq = TRUE;
 		return;
 	}
 
@@ -944,7 +929,7 @@ static void ieee802_1x_decapsulate_radius(hostapd *hapd, struct sta_info *sta)
 			       HOSTAPD_LEVEL_WARNING, "too short EAP packet "
 			       "received from authentication server");
 		free(eap);
-		sm->be_auth.eapNoReq = TRUE;
+		sm->eapNoReq = TRUE;
 		return;
 	}
 
@@ -977,7 +962,7 @@ static void ieee802_1x_decapsulate_radius(hostapd *hapd, struct sta_info *sta)
 		       HOSTAPD_LEVEL_DEBUG, "decapsulated EAP packet (code=%d "
 		       "id=%d len=%d) from RADIUS server: %s",
 		      hdr->code, hdr->identifier, ntohs(hdr->length), buf);
-	sm->be_auth.eapReq = TRUE;
+	sm->eapReq = TRUE;
 
 	free(sm->last_eap_radius);
 	sm->last_eap_radius = eap;
@@ -998,21 +983,13 @@ static void ieee802_1x_get_keys(hostapd *hapd, struct sta_info *sta,
 				      shared_secret_len);
 
 	if (keys) {
-		if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MINIMAL) && keys->send) {
-			size_t i;
-			printf("MS-MPPE-Send-Key (len=%lu):",
-			       (unsigned long) keys->send_len);
-			for (i = 0; i < keys->send_len; i++)
-				printf(" %02x", keys->send[i]);
-			printf("\n");
+		if (keys->send) {
+			wpa_hexdump_key(MSG_DEBUG, "MS-MPPE-Send-Key",
+					keys->send, keys->send_len);
 		}
-		if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MINIMAL) && keys->recv) {
-			size_t i;
-			printf("MS-MPPE-Recv-Key (len=%lu):",
-			       (unsigned long) keys->recv_len);
-			for (i = 0; i < keys->recv_len; i++)
-				printf(" %02x", keys->recv[i]);
-			printf("\n");
+		if (keys->recv) {
+			wpa_hexdump_key(MSG_DEBUG, "MS-MPPE-Recv-Key",
+					keys->recv, keys->recv_len);
 		}
 
 		if (keys->send && keys->recv) {
@@ -1245,7 +1222,7 @@ ieee802_1x_receive_auth(struct radius_msg *msg, struct radius_msg *req,
 		/* RFC 3580, Ch. 3.17 */
 		if (session_timeout_set && termination_action ==
 		    RADIUS_TERMINATION_ACTION_RADIUS_REQUEST) {
-			sm->reauth_timer.reAuthPeriod = session_timeout;
+			sm->reAuthPeriod = session_timeout;
 		} else if (session_timeout_set)
 			ap_sta_session_timeout(hapd, sta, session_timeout);
 
@@ -1255,10 +1232,13 @@ ieee802_1x_receive_auth(struct radius_msg *msg, struct radius_msg *req,
 				    shared_secret_len);
 		ieee802_1x_store_radius_class(hapd, sta, msg);
 		ieee802_1x_update_sta_identity(hapd, sta, msg);
-		if (sm->keyAvailable) {
-			pmksa_cache_add(hapd, sta, sm->eapol_key_crypt,
-					session_timeout_set ?
-					session_timeout : -1);
+		if (sm->keyAvailable &&
+		    wpa_auth_pmksa_add(sta->wpa_sm, sm->eapol_key_crypt,
+				       session_timeout_set ?
+				       session_timeout : -1, sm) == 0) {
+			hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_WPA,
+				       HOSTAPD_LEVEL_DEBUG,
+				       "Added PMKSA cache entry");
 		}
 		break;
 	case RADIUS_CODE_ACCESS_REJECT:
@@ -1285,7 +1265,7 @@ ieee802_1x_receive_auth(struct radius_msg *msg, struct radius_msg *req,
 
 	ieee802_1x_decapsulate_radius(hapd, sta);
 	if (override_eapReq)
-		sm->be_auth.eapReq = FALSE;
+		sm->eapReq = FALSE;
 
 	eapol_sm_step(sm);
 
@@ -1334,21 +1314,16 @@ void ieee802_1x_abort_auth(struct hostapd_data *hapd, struct sta_info *sta)
 }
 
 
-void ieee802_1x_set_port_enabled(hostapd *hapd, struct sta_info *sta,
-				 int enabled)
+#ifdef HOSTAPD_DUMP_STATE
+static void fprint_char(FILE *f, char c)
 {
-	if (!sta->eapol_sm)
-		return;
-
-	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
-		      "IEEE 802.1X: station " MACSTR " port %s\n",
-		      MAC2STR(sta->addr), enabled ? "enabled" : "disabled");
-	sta->eapol_sm->portEnabled = enabled ? TRUE : FALSE;
-	eapol_sm_step(sta->eapol_sm);
+	if (c >= 32 && c < 127)
+		fprintf(f, "%c", c);
+	else
+		fprintf(f, "<%02x>", c);
 }
 
 
-#ifdef HOSTAPD_DUMP_STATE
 void ieee802_1x_dump_state(FILE *f, const char *prefix, struct sta_info *sta)
 {
 	struct eapol_state_machine *sm = sta->eapol_sm;
@@ -1391,11 +1366,9 @@ static int ieee802_1x_rekey_broadcast(hostapd *hapd)
 		return -1;
 	}
 
-	if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MINIMAL)) {
-		hostapd_hexdump("IEEE 802.1X: New default WEP key",
-				hapd->default_wep_key,
-				hapd->conf->default_wep_key_len);
-	}
+	wpa_hexdump_key(MSG_DEBUG, "IEEE 802.1X: New default WEP key",
+			hapd->default_wep_key,
+			hapd->conf->default_wep_key_len);
 
 	return 0;
 }
@@ -1679,16 +1652,16 @@ int ieee802_1x_get_mib_sta(struct hostapd_data *hapd, struct sta_info *sta,
 			"dot1xAuthReAuthPeriod=%u\n"
 			"dot1xAuthReAuthEnabled=%s\n"
 			"dot1xAuthKeyTxEnabled=%s\n",
-			sm->auth_pae.state + 1,
-			sm->be_auth.state + 1,
-			sm->ctrl_dir.adminControlledDirections,
-			sm->ctrl_dir.operControlledDirections,
+			sm->auth_pae_state + 1,
+			sm->be_auth_state + 1,
+			sm->adminControlledDirections,
+			sm->operControlledDirections,
 			sm->authPortStatus,
 			sm->portControl,
-			sm->auth_pae.quietPeriod,
-			sm->be_auth.serverTimeout,
-			sm->reauth_timer.reAuthPeriod,
-			bool_txt(sm->reauth_timer.reAuthEnabled),
+			sm->quietPeriod,
+			sm->serverTimeout,
+			sm->reAuthPeriod,
+			bool_txt(sm->reAuthEnabled),
 			bool_txt(sm->keyTxEnabled));
 
 	/* dot1xAuthStatsTable */
@@ -1736,22 +1709,22 @@ int ieee802_1x_get_mib_sta(struct hostapd_data *hapd, struct sta_info *sta,
 			"dot1xAuthBackendOtherRequestsToSupplicant=%u\n"
 			"dot1xAuthBackendAuthSuccesses=%u\n"
 			"dot1xAuthBackendAuthFails=%u\n",
-			sm->auth_pae.authEntersConnecting,
-			sm->auth_pae.authEapLogoffsWhileConnecting,
-			sm->auth_pae.authEntersAuthenticating,
-			sm->auth_pae.authAuthSuccessesWhileAuthenticating,
-			sm->auth_pae.authAuthTimeoutsWhileAuthenticating,
-			sm->auth_pae.authAuthFailWhileAuthenticating,
-			sm->auth_pae.authAuthEapStartsWhileAuthenticating,
-			sm->auth_pae.authAuthEapLogoffWhileAuthenticating,
-			sm->auth_pae.authAuthReauthsWhileAuthenticated,
-			sm->auth_pae.authAuthEapStartsWhileAuthenticated,
-			sm->auth_pae.authAuthEapLogoffWhileAuthenticated,
-			sm->be_auth.backendResponses,
-			sm->be_auth.backendAccessChallenges,
-			sm->be_auth.backendOtherRequestsToSupplicant,
-			sm->be_auth.backendAuthSuccesses,
-			sm->be_auth.backendAuthFails);
+			sm->authEntersConnecting,
+			sm->authEapLogoffsWhileConnecting,
+			sm->authEntersAuthenticating,
+			sm->authAuthSuccessesWhileAuthenticating,
+			sm->authAuthTimeoutsWhileAuthenticating,
+			sm->authAuthFailWhileAuthenticating,
+			sm->authAuthEapStartsWhileAuthenticating,
+			sm->authAuthEapLogoffWhileAuthenticating,
+			sm->authAuthReauthsWhileAuthenticated,
+			sm->authAuthEapStartsWhileAuthenticated,
+			sm->authAuthEapLogoffWhileAuthenticated,
+			sm->backendResponses,
+			sm->backendAccessChallenges,
+			sm->backendOtherRequestsToSupplicant,
+			sm->backendAuthSuccesses,
+			sm->backendAuthFails);
 
 	/* dot1xAuthSessionStatsTable */
 	len += snprintf(buf + len, buflen - len,
@@ -1765,7 +1738,8 @@ int ieee802_1x_get_mib_sta(struct hostapd_data *hapd, struct sta_info *sta,
 			"dot1xAuthSessionTerminateCause=999\n"
 			"dot1xAuthSessionUserName=%s\n",
 			sta->acct_session_id_hi, sta->acct_session_id_lo,
-			sta->wpa_key_mgmt == WPA_KEY_MGMT_IEEE8021X ? 1 : 2,
+			wpa_auth_sta_key_mgmt(sta->wpa_sm) ==
+			WPA_KEY_MGMT_IEEE8021X ? 1 : 2,
 			(unsigned int) (time(NULL) - sta->acct_session_start),
 			sm->identity);
 
@@ -1782,7 +1756,11 @@ void ieee802_1x_finished(struct hostapd_data *hapd, struct sta_info *sta,
 	static const int dot11RSNAConfigPMKLifetime = 43200;
 
 	key = ieee802_1x_get_key_crypt(sta->eapol_sm, &len);
-	if (success && key) {
-		pmksa_cache_add(hapd, sta, key, dot11RSNAConfigPMKLifetime);
+	if (success && key &&
+	    wpa_auth_pmksa_add(sta->wpa_sm, key, dot11RSNAConfigPMKLifetime,
+			       sta->eapol_sm) == 0) {
+		hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_WPA,
+			       HOSTAPD_LEVEL_DEBUG,
+			       "Added PMKSA cache entry (IEEE 802.1X)");
 	}
 }
