@@ -12,9 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include "includes.h"
 
 #include "common.h"
 #include "sha1.h"
@@ -23,8 +21,14 @@
 #include "rc4.h"
 
 
-/* IN: 16-octet peer_challenge, 16-octet auth_challenge, 0..256-char username
- * OUT: 8-octet challenge */
+/**
+ * challenge_hash - ChallengeHash() - RFC 2759, Sect. 8.2
+ * @peer_challenge: 16-octet PeerChallenge (IN)
+ * @auth_challenge: 16-octet AuthChallenge (IN)
+ * @username: 0-to-256-char UserName (IN)
+ * @username_len: Length of username
+ * @challenge: 8-octet Challenge (OUT)
+ */
 static void challenge_hash(const u8 *peer_challenge, const u8 *auth_challenge,
 			   const u8 *username, size_t username_len,
 			   u8 *challenge)
@@ -45,13 +49,17 @@ static void challenge_hash(const u8 *peer_challenge, const u8 *auth_challenge,
 }
 
 
-/* IN: 0..256-unicode-char password
- * OUT: 16-octet password_hash */
+/**
+ * nt_password_hash - NtPasswordHash() - RFC 2759, Sect. 8.3
+ * @password: 0-to-256-unicode-char Password (IN)
+ * @password_len: Length of password
+ * @password_hash: 16-octet PasswordHash (OUT)
+ */
 void nt_password_hash(const u8 *password, size_t password_len,
 		      u8 *password_hash)
 {
 	u8 *buf;
-	int i;
+	size_t i, len;
 
 	/* Convert password into unicode */
 	buf = malloc(password_len * 2);
@@ -61,21 +69,30 @@ void nt_password_hash(const u8 *password, size_t password_len,
 	for (i = 0; i < password_len; i++)
 		buf[2 * i] = password[i];
 
-	md4(buf, password_len * 2, password_hash);
+	len = password_len * 2;
+	md4_vector(1, (const u8 **) &buf, &len, password_hash);
 	free(buf);
 }
 
 
-/* IN: 16-octet password_hash
- * OUT: 16-octet password_hash_hash */
+/**
+ * hash_nt_password_hash - HashNtPasswordHash() - RFC 2759, Sect. 8.4
+ * @password_hash: 16-octet PasswordHash (IN)
+ * @password_hash_hash: 16-octet PaswordHashHash (OUT)
+ */
 void hash_nt_password_hash(const u8 *password_hash, u8 *password_hash_hash)
 {
-	md4(password_hash, 16, password_hash_hash);
+	size_t len = 16;
+	md4_vector(1, &password_hash, &len, password_hash_hash);
 }
 
 
-/* IN: 8-octet challenge, 16-octet password_hash
- * OUT: 24-octet response */
+/**
+ * challenge_response - ChallengeResponse() - RFC 2759, Sect. 8.5
+ * @challenge: 8-octet Challenge (IN)
+ * @password_hash: 16-octet PasswordHash (IN)
+ * @response: 24-octet Response (OUT)
+ */
 void challenge_response(const u8 *challenge, const u8 *password_hash,
 			u8 *response)
 {
@@ -89,9 +106,16 @@ void challenge_response(const u8 *challenge, const u8 *password_hash,
 }
 
 
-/* IN: 16-octet auth_challenge, 16-octet peer_challenge, 0..256-char username,
- *     0..256-unicode-char password
- * OUT: 24-octet response */
+/**
+ * generate_nt_response - GenerateNTResponse() - RFC 2759, Sect. 8.1
+ * @auth_challenge: 16-octet AuthenticatorChallenge (IN)
+ * @peer_hallenge: 16-octet PeerChallenge (IN)
+ * @username: 0-to-256-char UserName (IN)
+ * @username_len: Length of username
+ * @password: 0-to-256-unicode-char Password (IN)
+ * @password_len: Length of password
+ * @response: 24-octet Response (OUT)
+ */
 void generate_nt_response(const u8 *auth_challenge, const u8 *peer_challenge,
 			  const u8 *username, size_t username_len,
 			  const u8 *password, size_t password_len,
@@ -107,14 +131,44 @@ void generate_nt_response(const u8 *auth_challenge, const u8 *peer_challenge,
 }
 
 
-/* IN: 0..256-unicode-char password, 24-octet nt_response,
- *     16-octet peer_challenge, 16-octer auth_challenge, 0..256-char username
- * OUT: 42-octet response */
-void generate_authenticator_response(const u8 *password, size_t password_len,
-				     const u8 *peer_challenge,
-				     const u8 *auth_challenge,
-				     const u8 *username, size_t username_len,
-				     const u8 *nt_response, u8 *response)
+/**
+ * generate_nt_response_pwhash - GenerateNTResponse() - RFC 2759, Sect. 8.1
+ * @auth_challenge: 16-octet AuthenticatorChallenge (IN)
+ * @peer_hallenge: 16-octet PeerChallenge (IN)
+ * @username: 0-to-256-char UserName (IN)
+ * @username_len: Length of username
+ * @password_hash: 16-octet PasswordHash (IN)
+ * @response: 24-octet Response (OUT)
+ */
+void generate_nt_response_pwhash(const u8 *auth_challenge,
+				 const u8 *peer_challenge,
+				 const u8 *username, size_t username_len,
+				 const u8 *password_hash,
+				 u8 *response)
+{
+	u8 challenge[8];
+
+	challenge_hash(peer_challenge, auth_challenge, username, username_len,
+		       challenge);
+	challenge_response(challenge, password_hash, response);
+}
+
+
+/**
+ * generate_authenticator_response_pwhash - GenerateAuthenticatorResponse() - RFC 2759, Sect. 8.7
+ * @password_hash: 16-octet PasswordHash (IN)
+ * @nt_response: 24-octet NT-Response (IN)
+ * @peer_challenge: 16-octet PeerChallenge (IN)
+ * @auth_challenge: 16-octet AuthenticatorChallenge (IN)
+ * @username: 0-to-256-char UserName (IN)
+ * @username_len: Length of username
+ * @response: 42-octet AuthenticatorResponse (OUT)
+ */
+void generate_authenticator_response_pwhash(
+	const u8 *password_hash,
+	const u8 *peer_challenge, const u8 *auth_challenge,
+	const u8 *username, size_t username_len,
+	const u8 *nt_response, u8 *response)
 {
 	static const u8 magic1[39] = {
 		0x4D, 0x61, 0x67, 0x69, 0x63, 0x20, 0x73, 0x65, 0x72, 0x76,
@@ -130,7 +184,7 @@ void generate_authenticator_response(const u8 *password, size_t password_len,
 		0x6E
 	};
 
-	u8 password_hash[16], password_hash_hash[16], challenge[8];
+	u8 password_hash_hash[16], challenge[8];
 	const unsigned char *addr1[3];
 	const size_t len1[3] = { 16, 24, sizeof(magic1) };
 	const unsigned char *addr2[3];
@@ -144,7 +198,6 @@ void generate_authenticator_response(const u8 *password, size_t password_len,
 	addr2[1] = challenge;
 	addr2[2] = magic2;
 
-	nt_password_hash(password, password_len, password_hash);
 	hash_nt_password_hash(password_hash, password_hash_hash);
 	sha1_vector(3, addr1, len1, response);
 
@@ -154,8 +207,39 @@ void generate_authenticator_response(const u8 *password, size_t password_len,
 }
 
 
-/* IN: 8-octet challege, 0..256-unicode-char password
- * OUT: 25-octet response */
+/**
+ * generate_authenticator_response - GenerateAuthenticatorResponse() - RFC 2759, Sect. 8.7
+ * @password: 0-to-256-unicode-char Password (IN)
+ * @password_len: Length of password
+ * @nt_response: 24-octet NT-Response (IN)
+ * @peer_challenge: 16-octet PeerChallenge (IN)
+ * @auth_challenge: 16-octet AuthenticatorChallenge (IN)
+ * @username: 0-to-256-char UserName (IN)
+ * @username_len: Length of username
+ * @response: 42-octet AuthenticatorResponse (OUT)
+ */
+void generate_authenticator_response(const u8 *password, size_t password_len,
+				     const u8 *peer_challenge,
+				     const u8 *auth_challenge,
+				     const u8 *username, size_t username_len,
+				     const u8 *nt_response, u8 *response)
+{
+	u8 password_hash[16];
+	nt_password_hash(password, password_len, password_hash);
+	generate_authenticator_response_pwhash(password_hash,
+					       peer_challenge, auth_challenge,
+					       username, username_len,
+					       nt_response, response);
+}
+
+
+/**
+ * nt_challenge_response - NtChallengeResponse() - RFC 2433, Sect. A.5
+ * @challenge: 8-octet Challenge (IN)
+ * @password: 0-to-256-unicode-char Password (IN)
+ * @password_len: Length of password
+ * @response: 24-octet Response (OUT)
+ */
 void nt_challenge_response(const u8 *challenge, const u8 *password,
 			   size_t password_len, u8 *response)
 {
@@ -165,8 +249,12 @@ void nt_challenge_response(const u8 *challenge, const u8 *password,
 }
 
 
-/* IN: 16-octet password_hash_hash and 24-octet nt_response
- * OUT: 16-octet master_key */
+/**
+ * get_master_key - GetMasterKey() - RFC 3079, Sect. 3.4
+ * @password_hash_hash: 16-octet PasswordHashHash (IN)
+ * @nt_response: 24-octet NTResponse (IN)
+ * @master_key: 16-octet MasterKey (OUT)
+ */
 void get_master_key(const u8 *password_hash_hash, const u8 *nt_response,
 		    u8 *master_key)
 {
@@ -188,9 +276,14 @@ void get_master_key(const u8 *password_hash_hash, const u8 *nt_response,
 }
 
 
-/* IN: 16-octet master_key
- * IN: session_key_len, is_send, is_server
- * OUT: session_key (8..16 octets) */
+/**
+ * get_asymetric_start_key - GetAsymetricStartKey() - RFC 3079, Sect. 3.4
+ * @master_key: 16-octet MasterKey (IN)
+ * @session_key: 8-to-16 octet SessionKey (OUT)
+ * @session_key_len: SessionKeyLength (Length of session_key)
+ * @is_send: IsSend (IN, BOOLEAN)
+ * @is_server: IsServer (IN, BOOLEAN)
+ */
 void get_asymetric_start_key(const u8 *master_key, u8 *session_key,
 			     size_t session_key_len, int is_send,
 			     int is_server)
@@ -253,8 +346,13 @@ void get_asymetric_start_key(const u8 *master_key, u8 *session_key,
 
 #define PWBLOCK_LEN 516
 
-/* IN: 0..256-unicode-char password, 16-octet password_hash
- * OUT: 516-byte pw_block */
+/**
+ * encrypt_pw_block_with_password_hash - EncryptPwBlobkWithPasswordHash() - RFC 2759, Sect. 8.10
+ * @password: 0-to-256-unicode-char Password (IN)
+ * @password_len: Length of password
+ * @password_hash: 16-octet PasswordHash (IN)
+ * @pw_block: 516-byte PwBlock (OUT)
+ */
 static void encrypt_pw_block_with_password_hash(
 	const u8 *password, size_t password_len,
 	const u8 *password_hash, u8 *pw_block)
@@ -270,17 +368,18 @@ static void encrypt_pw_block_with_password_hash(
 	for (i = 0; i < password_len; i++)
 		pw_block[offset + i * 2] = password[i];
 	pos = &pw_block[2 * 256];
-	*pos++ = (password_len * 2) & 0xff;
-	*pos = ((password_len * 2) >> 8) & 0xff;
+	WPA_PUT_LE16(pos, password_len * 2);
 	rc4(pw_block, PWBLOCK_LEN, password_hash, 16);
 }
 
 
-/* NewPasswordEncryptedWithOldNtPasswordHash() routine defined in [RFC2759],
- * Section 8.9.
- * IN: 0..256-unicode-char new_password,
- *     0..256-unicode-char old_password
- * OUT: 516-octet encrypted_pw_block
+/**
+ * new_password_encrypted_with_old_nt_password_hash - NewPasswordEncryptedWithOldNtPasswordHash() - RFC 2759, Sect. 8.9
+ * @new_password: 0-to-256-unicode-char NewPassword (IN)
+ * @new_password_len: Length of new_password
+ * @old_password: 0-to-256-unicode-char OldPassword (IN)
+ * @old_password_len: Length of old_password
+ * @encrypted_pw_block: 516-octet EncryptedPwBlock (OUT)
  */
 void new_password_encrypted_with_old_nt_password_hash(
 	const u8 *new_password, size_t new_password_len,
@@ -295,8 +394,12 @@ void new_password_encrypted_with_old_nt_password_hash(
 }
 
 
-/* IN: 16-octer password_hash, 16-octet block
- * OUT: 16-octer cypher */
+/**
+ * nt_password_hash_encrypted_with_block - NtPasswordHashEncryptedWithBlock() - RFC 2759, Sect 8.13
+ * @password_hash: 16-octer PasswordHash (IN)
+ * @block: 16-octet Block (IN)
+ * @cypher: 16-octer Cypher (OUT)
+ */
 static void nt_password_hash_encrypted_with_block(const u8 *password_hash,
 						  const u8 *block,
 						  u8 *cypher)
@@ -306,11 +409,13 @@ static void nt_password_hash_encrypted_with_block(const u8 *password_hash,
 }
 
 
-/* OldNtPasswordHashEncryptedWithNewNtPasswordHash() routine defined in
- * [RFC2759], Section 8.12.
- * IN: 0..256-unicode-char new_password,
- *     0..256-unicode-char old_password
- * OUT: 16-octet encrypted_password_hash
+/**
+ * old_nt_password_hash_encrypted_with_new_nt_password_hash - OldNtPasswordHashEncryptedWithNewNtPasswordHash() - RFC 2759, Sect. 8.12
+ * @new_password: 0-to-256-unicode-char NewPassword (IN)
+ * @new_password_len: Length of new_password
+ * @old_password: 0-to-256-unicode-char OldPassword (IN)
+ * @old_password_len: Length of old_password
+ * @encrypted_password_ash: 16-octet EncryptedPasswordHash (OUT)
  */
 void old_nt_password_hash_encrypted_with_new_nt_password_hash(
 	const u8 *new_password, size_t new_password_len,
@@ -328,6 +433,9 @@ void old_nt_password_hash_encrypted_with_new_nt_password_hash(
 
 
 #ifdef TEST_MAIN_MS_FUNCS
+
+#include "rc4.c"
+
 int main(int argc, char *argv[])
 {
 	/* Test vector from RFC2759 example */
