@@ -1,3 +1,17 @@
+/*
+ * WPA Supplicant / shared MSCHAPV2 helper functions / RFC 2433 / RFC 2759
+ * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ *
+ * Alternatively, this software may be distributed under the terms of BSD
+ * license.
+ *
+ * See README and COPYING for more details.
+ */
+
 #ifndef MS_FUNCS_H
 #define MS_FUNCS_H
 
@@ -5,11 +19,21 @@ void generate_nt_response(const u8 *auth_challenge, const u8 *peer_challenge,
 			  const u8 *username, size_t username_len,
 			  const u8 *password, size_t password_len,
 			  u8 *response);
+void generate_nt_response_pwhash(const u8 *auth_challenge,
+				 const u8 *peer_challenge,
+				 const u8 *username, size_t username_len,
+				 const u8 *password_hash,
+				 u8 *response);
 void generate_authenticator_response(const u8 *password, size_t password_len,
 				     const u8 *peer_challenge,
 				     const u8 *auth_challenge,
 				     const u8 *username, size_t username_len,
 				     const u8 *nt_response, u8 *response);
+void generate_authenticator_response_pwhash(
+	const u8 *password_hash,
+	const u8 *peer_challenge, const u8 *auth_challenge,
+	const u8 *username, size_t username_len,
+	const u8 *nt_response, u8 *response);
 void nt_challenge_response(const u8 *challenge, const u8 *password,
 			   size_t password_len, u8 *response);
 
