@@ -12,13 +12,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
-#include <netinet/in.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
+#include "includes.h"
 #include <net/if.h>
 
 #include "common.h"
@@ -47,6 +41,12 @@ struct radius_session {
 	size_t eapKeyDataLen, eapReqDataLen;
 	Boolean eapSuccess, eapRestart, eapFail, eapResp, eapReq, eapNoReq;
 	Boolean portEnabled, eapTimeout;
+
+	struct radius_msg *last_msg;
+	char *last_from_addr;
+	int last_from_port;
+	struct sockaddr_storage last_from;
+	socklen_t last_fromlen;
 };
 
 struct radius_client {
@@ -65,7 +65,6 @@ struct radius_client {
 struct radius_server_data {
 	int auth_sock;
 	struct radius_client *clients;
-	struct radius_server_session *sessions;
 	unsigned int next_sess_id;
 	void *hostapd_conf;
 	int num_sess;
@@ -153,6 +152,11 @@ static void radius_server_session_free(struct radius_server_data *data,
 	free(sess->eapKeyData);
 	free(sess->eapReqData);
 	eap_sm_deinit(sess->eap);
+	if (sess->last_msg) {
+		radius_msg_free(sess->last_msg);
+		free(sess->last_msg);
+	}
+	free(sess->last_from_addr);
 	free(sess);
 	data->num_sess--;
 }
@@ -456,6 +460,13 @@ static int radius_server_request(struct radius_server_data *data,
 		resp_id = 0;
 	}
 
+	/* FIX: if Code is Request, Success, or Failure, send Access-Reject;
+	 * RFC3579 Sect. 2.6.2.
+	 * Include EAP-Response/Nak with no preferred method if
+	 * code == request.
+	 * If code is not 1-4, discard the packet silently.
+	 * Or is this already done by the EAP state machine? */
+
 	eap_set_eapRespData(sess->eap, eap, eap_len);
 	free(eap);
 	eap = NULL;
@@ -476,6 +487,18 @@ static int radius_server_request(struct radius_server_data *data,
 			sess->eapReqData = (u8 *) hdr;
 			sess->eapReqDataLen = sizeof(*hdr);
 		}
+	} else if (eap_sm_method_pending(sess->eap)) {
+		if (sess->last_msg) {
+			radius_msg_free(sess->last_msg);
+			free(sess->last_msg);
+		}
+		sess->last_msg = msg;
+		sess->last_from_port = from_port;
+		free(sess->last_from_addr);
+		sess->last_from_addr = strdup(from_addr);
+		sess->last_fromlen = fromlen;
+		memcpy(&sess->last_from, from, fromlen);
+		return -2;
 	} else {
 		RADIUS_DEBUG("No EAP data from the state machine - ignore this"
 			     " Access-Request silently (assuming it was a "
@@ -596,8 +619,9 @@ static void radius_server_receive_auth(int sock, void *eloop_ctx,
 		goto fail;
 	}
 
-	radius_server_request(data, msg, (struct sockaddr *) &from, fromlen,
-			      client, abuf, from_port);
+	if (radius_server_request(data, msg, (struct sockaddr *) &from,
+				  fromlen, client, abuf, from_port) == -2)
+		return; /* msg was stored with the session */
 
 fail:
 	if (msg) {
@@ -1050,6 +1074,7 @@ static int radius_server_get_eap_user(void *ctx, const u8 *identity,
 		memcpy(user->password, eap_user->password,
 		       eap_user->password_len);
 		user->password_len = eap_user->password_len;
+		user->password_hash = eap_user->password_hash;
 	}
 	user->force_version = eap_user->force_version;
 
@@ -1065,3 +1090,43 @@ static struct eapol_callbacks radius_server_eapol_cb =
 	.set_eapKeyData = radius_server_set_eapKeyData,
 	.get_eap_user = radius_server_get_eap_user,
 };
+
+
+void radius_server_eap_pending_cb(struct radius_server_data *data, void *ctx)
+{
+	struct radius_client *cli;
+	struct radius_session *s, *sess = NULL;
+	struct radius_msg *msg;
+
+	if (data == NULL)
+		return;
+
+	for (cli = data->clients; cli; cli = cli->next) {
+		for (s = cli->sessions; s; s = s->next) {
+			if (s->eap == ctx && s->last_msg) {
+				sess = s;
+				break;
+			}
+			if (sess)
+				break;
+		}
+		if (sess)
+			break;
+	}
+
+	if (sess == NULL)
+		return;
+
+	msg = sess->last_msg;
+	sess->last_msg = NULL;
+	eap_sm_pending_cb(sess->eap);
+	if (radius_server_request(data, msg,
+				  (struct sockaddr *) &sess->last_from,
+				  sess->last_fromlen, cli,
+				  sess->last_from_addr,
+				  sess->last_from_port) == -2)
+		return; /* msg was stored with the session */
+
+	radius_msg_free(msg);
+	free(msg);
+}
