@@ -9,6 +9,7 @@ struct radius_server_conf {
 	void *hostapd_conf;
 	void *eap_sim_db_priv;
 	void *ssl_ctx;
+	int ipv6;
 };
 
 
@@ -22,6 +23,8 @@ void radius_server_deinit(struct radius_server_data *data);
 int radius_server_get_mib(struct radius_server_data *data, char *buf,
 			  size_t buflen);
 
+void radius_server_eap_pending_cb(struct radius_server_data *data, void *ctx);
+
 #else /* RADIUS_SERVER */
 
 static inline struct radius_server_data *
@@ -40,6 +43,11 @@ static inline int radius_server_get_mib(struct radius_server_data *data,
 	return 0;
 }
 
+static inline void
+radius_server_eap_pending_cb(struct radius_server_data *data, void *ctx)
+{
+}
+
 #endif /* RADIUS_SERVER */
 
 #endif /* RADIUS_SERVER_H */
