@@ -13,12 +13,7 @@
  * See README and COPYING for more details.
  */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <netinet/in.h>
-#include <sys/types.h>
-#include <sys/socket.h>
+#include "includes.h"
 
 #include "hostapd.h"
 #include "sta_info.h"
@@ -29,6 +24,7 @@
 #include "radius.h"
 #include "eapol_sm.h"
 #include "wpa.h"
+#include "preauth.h"
 #include "radius_client.h"
 #include "driver.h"
 #include "hostap_common.h"
@@ -125,14 +121,14 @@ void ap_free_sta(hostapd *hapd, struct sta_info *sta)
 	eloop_cancel_timeout(ap_handle_timer, hapd, sta);
 
 	ieee802_1x_free_station(sta);
-	wpa_free_station(sta);
+	wpa_auth_sta_deinit(sta->wpa_sm);
+	rsn_preauth_free_station(hapd, sta);
 	radius_client_flush_auth(hapd->radius, sta->addr);
 
 	if (sta->last_assoc_req)
 		free(sta->last_assoc_req);
 
 	free(sta->challenge);
-	free(sta->wpa_ie);
 
 	free(sta);
 }
@@ -255,7 +251,7 @@ void ap_handle_timer(void *eloop_ctx, void *timeout_ctx)
 		break;
 	case STA_DISASSOC:
 		sta->flags &= ~WLAN_STA_ASSOC;
-		ieee802_1x_set_port_enabled(hapd, sta, 0);
+		ieee802_1x_notify_port_enabled(sta->eapol_sm, 0);
 		if (!sta->acct_terminate_cause)
 			sta->acct_terminate_cause =
 				RADIUS_ACCT_TERMINATE_CAUSE_IDLE_TIMEOUT;
